<?php

/*
 * Service class for the Workflow Process flow
 *
 * @copyright   Copyright (c) 2016, Nugget Solutions, Inc
 * @license     http://opensource.org/licenses/gpl-2.0.php GNU Public License
 * @since       2.0
 *
 */

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) )
   exit;

/*
 * OW_Inbox Class
 *
 * @since 2.0
 */

class OW_Process_Flow {
   /*
    * Set things up.
    *
    * @since 2.0
    */

   public function __construct() {
      add_action( 'wp_trash_post', array( $this, 'when_post_trash_delete' ) );

      add_action( 'wp_ajax_check_for_claim_ajax', array( $this, 'check_for_claim_ajax' ) );
      add_action( 'wp_ajax_claim_process', array( $this, 'claim_process' ) );
      add_action( 'wp_ajax_reassign_process', array( $this, 'reassign_process' ) );
      add_action( 'wp_ajax_get_users_in_step', array( $this, 'get_users_in_step' ) );
      add_action( 'wp_ajax_get_pre_next_steps', array( $this, 'get_pre_next_steps' ) );
      add_action( 'wp_ajax_change_workflow_status_to_complete', array( $this, 'change_workflow_status_to_complete' ) );
      add_action( 'wp_ajax_change_workflow_status_to_cancelled', array( $this, 'change_workflow_status_to_cancelled' ) );
      add_action( 'wp_ajax_submit_post_to_step', array( $this, 'submit_post_to_step' ) );
      add_action( 'wp_ajax_get_step_status_by_history_id', array( $this, 'get_step_status_by_history_id' ) );
      add_action( 'wp_ajax_get_step_status_by_step_id', array( $this, 'get_step_status_by_step_id' ) );
      add_action( 'save_post', array( $this, 'update_oasis_is_in_workflow' ), 10, 1 );
      add_action( 'wp_ajax_exit_post_from_workflow', array( $this, 'exit_post_from_workflow' ) );
      add_action( 'wp_ajax_multi_abort_from_workflow', array( $this, 'multi_abort_from_workflow' ) );
      add_action( 'wp_ajax_get_post_publish_date_edit_format', array( $this, 'get_post_publish_date_edit_format' ) );
      add_action( 'redirect_post_location', array( $this, 'redirect_after_signoff' ) );
      add_action( 'wp_ajax_submit_post_to_workflow_pre', array( $this, 'submit_post_to_workflow_pre' ) );
      add_action( 'wp_ajax_sign_off_post_from_workflow_pre', array( $this, 'sign_off_post_from_workflow_pre' ) );
      }

   /*
    * get all the users who have atleast one post in their inbox
    *
    * @return mixed user list
    *
    * @since 2.0
    */

   public function get_assigned_users() {
      global $wpdb;

      $sql = "SELECT distinct USERS.ID, USERS.display_name FROM
		(SELECT U1.ID, U1.display_name FROM {$wpdb->users} AS U1
		LEFT JOIN " . OW_Utility::instance()->get_action_history_table_name() . " AS AH ON U1.ID = AH.assign_actor_id
		WHERE AH.action_status = 'assignment'
		UNION
		SELECT U2.ID, U2.display_name FROM {$wpdb->users} AS U2
		LEFT JOIN " . OW_Utility::instance()->get_action_table_name() . " AS A ON U2.ID = A.actor_id
					WHERE A.review_status = 'assignment') USERS
					ORDER BY USERS.DISPLAY_NAME ";

      $result = $wpdb->get_results( $sql );
      return $result;
   }

   /*
    * get the assigned posts to a particular user
    *
    * @param int|null $post_id
    * @param int|null $user_id
    * @param mixed $return_format it could be rows or just a single row
    *
    * @since 2.0
    */

   // TODO : change the function name to get_assigned_tasks
   public function get_assigned_post( $post_id = null, $user_id = null, $return_format = "rows" ) {
      global $wpdb;

      if ( !empty( $post_id ) ) {
         $post_id = intval( $post_id );
      }

      if ( !empty( $user_id ) ) {
         $user_id = intval( $user_id );
      }

      // use white list approach to set order by clause
      $order_by = array(
          'post_title' => 'post_title',
          'post_type' => 'post_type',
          'post_author' => 'post_author',
          'due_date' => 'due_date'
      );

      $sort_order = array(
          'asc' => 'ASC',
          'desc' => 'DESC',
      );

      // default order by
      $order_by_column = " ORDER BY A.due_date, posts.post_title"; // default order by column
      // if user provided any order by and order input, use that
      if ( isset( $_GET['orderby'] ) && $_GET['orderby'] ) {
         // sanitize data
         $user_provided_order_by = sanitize_text_field( $_GET['orderby'] );
         $user_provided_order = sanitize_text_field( $_GET['order'] );
         if ( array_key_exists( $user_provided_order_by, $order_by ) ) {
            $order_by_column = " ORDER BY " . $order_by[$user_provided_order_by] . " " . $sort_order[$user_provided_order];
         }
      }

      // added a left outer join to priority, since it may or may not be present.

      $sql = "SELECT A.*, postmeta.meta_value AS priority, B.review_status, B.actor_id,
      			B.next_assign_actors, B.step_id as review_step_id, B.action_history_id, B.update_datetime,
      			posts.post_title, users.display_name as post_author, posts.post_type
      			FROM " . OW_Utility::instance()->get_action_history_table_name() . " A
      			LEFT OUTER JOIN  " . OW_Utility::instance()->get_action_table_name() . " B ON A.ID = B.action_history_id
      			AND B.review_status = 'assignment'
					LEFT JOIN {$wpdb->posts} AS posts ON posts.ID = A.post_id
					LEFT OUTER JOIN {$wpdb->postmeta} AS postmeta ON postmeta.post_id = A.post_id
					AND postmeta.meta_key = 'ow_task_priority'
					LEFT JOIN {$wpdb->base_prefix}users AS users ON users.ID = posts.post_author
					WHERE 1 = 1 AND A.action_status = 'assignment' ";

      // generate the where clause and get the results
      if ( $post_id ) {
         $where_clause = "AND (assign_actor_id = %d OR actor_id = %d) AND A.post_id = %d " . $order_by_column;
         if ( $return_format == "rows" ) {
            $result = $wpdb->get_results( $wpdb->prepare( $sql . $where_clause, $user_id, $user_id, $post_id ) );
         } else {
            $result = $wpdb->get_row( $wpdb->prepare( $sql . $where_clause, $user_id, $user_id, $post_id ) );
         }
      } else if ( isset( $user_id ) ) {
         $where_clause = "AND assign_actor_id = %d OR actor_id = %d  " . $order_by_column;
         if ( $return_format == "rows" ) {
            $result = $wpdb->get_results( $wpdb->prepare( $sql . $where_clause, $user_id, $user_id ) );
         } else {
            $result = $wpdb->get_row( $wpdb->prepare( $sql . $where_clause, $user_id, $user_id ) );
         }
      } else {
         $where_clause = $order_by_column;
         if ( $return_format == "rows" ) {
            $result = $wpdb->get_results( $sql . $where_clause );
         } else {
            $result = $wpdb->get_row( $sql . $where_clause );
         }
      }

      return $result;
   }

   /*
    * Get count of assigned tasks
    *
    * @return int count of assigned tasks for the current user OR the passed in user.
    *
    * @since 2.0
    */

   public function get_assigned_post_count() {
      global $wpdb;
      $selected_user = isset( $_GET['user'] ) ? intval( $_GET["user"] ) : get_current_user_id();

      $assigned_tasks = $wpdb->get_var(
              $wpdb->prepare( "SELECT count(1) FROM " . OW_Utility::instance()->get_action_history_table_name() . " AH
                 LEFT OUTER JOIN " . OW_Utility::instance()->get_action_table_name() . " A
                 ON AH.ID = A.action_history_id
                 AND A.review_status = 'assignment'
                 WHERE AH.action_status = 'assignment'
                 AND (AH.assign_actor_id = %d OR A.actor_id = %d)",
                 $selected_user, $selected_user ) );

      return $assigned_tasks;
   }

   /**
    * Get all post_ids that are currently in a workflow.
    *
    * @return mixed array of post_ids
    *
    * @since 2.0
    */

   public function get_all_assigned_posts() {
      global $wpdb;
      $post_id_array = "";

      // anything which the action_status of "assignment" is currently in workflow and assigned.
      $sql = "SELECT DISTINCT(A.post_id) as post_id FROM
							(SELECT * FROM " . OW_Utility::instance()->get_action_history_table_name() . " WHERE action_status = 'assignment') as A
							LEFT OUTER JOIN
							(SELECT * FROM " . OW_Utility::instance()->get_action_table_name() . " WHERE review_status = 'assignment') as B
							ON A.ID = B.action_history_id order by A.due_date";

      // create a post_id array from the result set
      $assign_posts = $wpdb->get_results( $sql );
      if ( $assign_posts ) {
         foreach ( $assign_posts as $post ) {
            $post_id_array[] = $post->post_id;
         }
      }
      return $post_id_array;
   }

   /**
    * Get all the submitted articles.
    *
    * Get all the posts/pages/custom post types that are currently in a workflow.
    * It calls get_all_assigned_posts to get all assigned post_ids.
    * And then gets the details on those posts_ids.
    *
    * @param string $post_type specific post type otherwise "all"
    *
    * @return mixed array of posts
    *
    * @since 2.0
    */

   public function get_submitted_articles( $post_type = 'all' ) {
      global $wpdb;
      $post_type = sanitize_text_field( $post_type );

      // get an array of all the assigned posts
      $assign_post_ids = $this->get_all_assigned_posts();
      $assign_post_ids = ( $assign_post_ids ) ? $assign_post_ids : array( -1 );
      $submited_posts = null;

      // get post details
      if ( $post_type === "all" ) {
         $sql = "SELECT posts.ID, posts.post_author, posts.post_title, posts.post_type, posts.post_date FROM " . $wpdb->posts . " as posts WHERE ID IN (" . implode( $assign_post_ids, "," ) . ") ORDER BY ID DESC";
         $submited_posts = $wpdb->get_results( $sql );
      } else {
         $sql = "SELECT posts.ID, posts.post_author, posts.post_title, posts.post_type, posts.post_date FROM " . $wpdb->posts . " as posts WHERE post_type = %s AND ID IN (" . implode( $assign_post_ids, "," ) . ") ORDER BY ID DESC";
         $submited_posts = $wpdb->get_results( $wpdb->prepare( $sql, $post_type ) );
      }

      return $submited_posts;
   }

   /**
    * Get all the un-submitted articles.
    *
    * Get all the posts/pages/custom post types which are not published
    * and are not in any workflow.
    *
    * @param string $post_type specific post type otherwise "all"
    *
    * @return mixed array of posts
    * @since 2.0
    */

   public function get_unsubmitted_articles( $post_type = 'all' ) {
      global $wpdb;
      $post_type = sanitize_text_field( $post_type );

      foreach ( get_post_stati( array( 'show_in_admin_status_list' => true ) ) as $key => $status ) {
         if ( $status != 'publish' && $status != 'trash' ) { //not published
            $auto_submit_stati[$key] = "'" . esc_sql( $status ) . "'";
         }
      }
      $auto_submit_stati_list = join( ",", $auto_submit_stati );
      $unsubmitted_posts = null;

      // get all posts which are not published and are not in workflow
      if ( $post_type === "all" ) {
         $unsubmitted_posts = $wpdb->get_results( "SELECT distinct posts.ID, posts.post_author, posts.post_title, posts.post_type, posts.post_date FROM {$wpdb->prefix}posts posts
			WHERE posts.post_status in (" . $auto_submit_stati_list . ")
			AND
			(NOT EXISTS (SELECT * from {$wpdb->prefix}postmeta postmeta1 WHERE postmeta1.meta_key = 'oasis_is_in_workflow' and posts.ID = postmeta1.post_id) OR
			EXISTS (SELECT * from {$wpdb->prefix}postmeta postmeta2 WHERE postmeta2.meta_key = 'oasis_is_in_workflow' AND postmeta2.meta_value = '0' and posts.ID = postmeta2.post_id))
			order by post_modified_gmt" );
      } else {
         $sql = "SELECT distinct posts.ID, posts.post_author, posts.post_title, posts.post_type, posts.post_date FROM {$wpdb->prefix}posts posts
			WHERE post_type = %s AND posts.post_status in (" . $auto_submit_stati_list . ")
			AND
			(NOT EXISTS (SELECT * from {$wpdb->prefix}postmeta postmeta1 WHERE postmeta1.meta_key = 'oasis_is_in_workflow' and posts.ID = postmeta1.post_id) OR
			EXISTS (SELECT * from {$wpdb->prefix}postmeta postmeta2 WHERE postmeta2.meta_key = 'oasis_is_in_workflow' AND postmeta2.meta_value = '0' and posts.ID = postmeta2.post_id))
			order by post_modified_gmt";

         $unsubmitted_posts = $wpdb->get_results( $wpdb->prepare( $sql, $post_type ) );
      }

      return $unsubmitted_posts;
   }

   /*
    * get comments count
    *
    * @param int $action_history_id
    * @param boolean $is_inbox_comment - variable to show the comments blurb on the inbox
    * @param boolean|int $post_id - sign off comments for post_id
    */

   public function get_comment_count( $action_history_id, $is_inbox_comment = FALSE, $post_id = FALSE ) {

      if ( !empty( $action_history_id ) ) {
         $action_history_id = intval( $action_history_id );
      }

      if ( !empty( $post_id ) ) {
         $post_id = intval( $post_id );
      }

      if ( !empty( $is_inbox_comment ) ) {
         $is_inbox_comment = sanitize_text_field( $is_inbox_comment );
      }

      $i = 0;

      // in case of inbox comments, we need to count all the previous comments as well
      if ( $is_inbox_comment && $post_id > 0 ) {
         $action_history_ids = array();
         // get the comments from the assignment/publish steps
         $results = $this->get_assignment_comment_for_post( $post_id );
         if ( $results ) {
            foreach ( $results as $result ) {
               $action_history_ids[] = $result->ID;
               if ( !empty( $result->comment ) ) {
                  $comments = json_decode( $result->comment );
                  // Display comment count if comment index is not null
                  if ( !empty( $comments[0]->comment ) ) {
                     $i = $i + count( $comments );
                  }
               }
            }
         }
         // get reassigned comments from the review steps
         if ( !empty( $action_history_ids ) ) {
            $results = $this->get_reassigned_comments_for_review_steps( $action_history_ids );
            if ( $results ) {
               foreach ( $results as $result ) {
                  if ( !empty( $result->comments ) ) {
                     $i ++;
                  }
               }
            }
         }
      } else { // non inbox page, could be history page
         $ow_history_service = new OW_History_Service();
         $action_history = $ow_history_service->get_action_history_by_id( $action_history_id );
         if ( $action_history ) {
            $comments = json_decode( $action_history->comment );
            if ( $comments ) {
               foreach ( $comments as $comment ) {
                  if ( $comment->comment ) {
                     $i ++;
                  }
               }
            }
         }
      }

      return $i;
   }

   /*
    * get the comments for assignment/publish steps
    *
    * @param int $post_id - post_id for which to get the sign off comments
    * @return mixed comments
    */

   public function get_assignment_comment_for_post( $post_id ) {
      global $wpdb;

      // sanitize the data
      $post_id = intval( $post_id );

      $table = OW_Utility::instance()->get_action_history_table_name();
      $sql = "SELECT ID, comment  FROM $table
			WHERE post_id = '%d'
			AND action_status NOT IN ('submitted', 'claimed', 'claim_cancel')
			GROUP BY from_id
      	ORDER BY ID DESC";

      return $wpdb->get_results( $wpdb->prepare( $sql, $post_id ) );
   }

   /*
    * get reassigned comments for the review steps for all the action_history_ids
    *
    * @param array $action_history_ids array of action history ids
    * @return mixed review_action data for reassigned actions
    */

   public function get_reassigned_comments_for_review_steps( $action_history_ids ) {
      global $wpdb;

      // sanitize the values
      $action_history_ids = array_map( 'esc_attr', $action_history_ids );

      $table = OW_Utility::instance()->get_action_table_name();

      $imploded_action_history_ids = implode( ",", $action_history_ids );
      $action_history_condition = "action_history_id IN (" . $imploded_action_history_ids . ")";
      $sql = "SELECT *  FROM $table
		WHERE review_status = 'reassigned'
		AND " . $action_history_condition;

      return $wpdb->get_results( $sql );
   }

   /*
    * If there are more than one user assigned to the same task as part of "assignment" or "publish" step
    * We need to show the "Claim" link.
    * @param int $action_history_id
    * @return boolean true, if claim is required, false if not.
    */

   public function check_for_claim( $action_history_id ) {
      global $wpdb;

      // sanitize the data
      $action_history_id = intval( $action_history_id );

      $ow_history_service = new OW_History_Service();
      $action_history = $ow_history_service->get_action_history_by_id( $action_history_id );
      $rows = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM " . OW_Utility::instance()->get_action_history_table_name() .
                      " WHERE action_status = 'assignment' AND post_id = %d", $action_history->post_id ) );

      if ( count( $rows ) > 1 ) { // more than one rows of assignment, return true
         return true;
      }
      return false; // looks like there is only one assignment task, so no "Claim" button needed.
   }

   /*
    * AJAX version of check_for_claim
    */

   public function check_for_claim_ajax() {

      check_ajax_referer( 'owf_check_claim_nonce', 'security' );

      $action_history_id = intval( $_POST["history_id"] );

      // check if we need to show the claim button or not.
      if ( $this->check_for_claim( $action_history_id ) ) {
         echo "true";
      } else {
         echo "false";
      }
      die();
   }

   /*
    * Actual Claim process - ajax function
    * Checks for claim, if true, adds a record in the history table for the claim action
    * deletes all the step emails which are not applicable anymore
    * send the actor who claimed the article about the assignment email and reminder emails (if any)
    * notify all other users in that step about the "task being claimed".
    *
    * @return int id of the newly added history object for the claim process
    *
    * @since 2.0
    */

   public function claim_process() {
      global $wpdb;

      check_ajax_referer( 'owf_claim_process_ajax_nonce', 'security' );

      $action_history_id = intval( $_POST["actionid"] );

      $action_history_table = OW_Utility::instance()->get_action_history_table_name();
      $ow_history_service = new OW_History_Service();
      $action_history = $ow_history_service->get_action_history_by_id( $action_history_id );
      $ow_email = new OW_Email( );
      $post_title = "";
      if ( $this->check_for_claim( $action_history_id ) ) { // First check if "claim" is applicable or not
         $action_histories = $ow_history_service->get_action_history_by_status( "assignment", $action_history->post_id );
         foreach ( $action_histories as $action ) { // for all the history ids, only one will be "claimed", rest need to be "unclaimed" OR claim_cancelled.
            if ( $post_title == "" ) {
               $post_title = stripcslashes( get_post( $action->post_id )->post_title );
            }
            if ( $action_history_id == $action->ID ) { // this is a match, so claim
               // add claim action to history table
               $claim_history_data = (array) $action;
               unset( $claim_history_data["ID"] ); //unset the id, since we will get a new ID after insert
               $claim_history_data["action_status"] = "assignment";
               $claim_history_data["from_id"] = $action->ID;
               $claim_history_data["create_datetime"] = current_time( 'mysql' );
               if ( empty( $action->due_date ) ) {
                  unset( $claim_history_data["due_date"] );
               }
               if ( empty( $action->reminder_date ) ) {
                  unset( $claim_history_data["reminder_date"] );
               }
               if ( empty( $action->reminder_date_after ) ) {
                  unset( $claim_history_data["reminder_date_after"] );
               }
               $new_history_id = OW_Utility::instance()->insert_to_table( $action_history_table, $claim_history_data );
               // delete reminder emails, since the assignment is now claimed
               $ow_email->delete_step_email( $action->ID, $action->assign_actor_id );

               // send mail to the actor about the assignment and add email reminders, if any
               $ow_email->send_step_email( $new_history_id );

               $data["action_status"] = "claimed";
            } else {
               $data["action_status"] = "claim_cancel";

               // send email to other users, saying that the article has been removed from their inbox, since it was claimed by another user
               $ow_email->notify_users_on_task_claimed( $action->assign_actor_id, $action->post_id );
               $ow_email->delete_step_email( $action->ID, $action->assign_actor_id );
            }
            $wpdb->update( $action_history_table, $data, array( "ID" => $action->ID ) );
         }
      }
      echo trim( $new_history_id );
      die();
   }

   /*
    * Reassign process
    */

   public function reassign_process() {
      global $wpdb;
      check_ajax_referer( 'owf_reassign_ajax_nonce', 'security' );

      if ( ! current_user_can( 'ow_reassign_task' ) ) {
      	wp_die( __( 'You are not allowed to reassign tasks.' ) );
      }

      $current_user = (sanitize_text_field( $_POST["task_user"] ) != "") ? intval( sanitize_text_field( $_POST["task_user"] ) ) : get_current_user_id();
      $action_history_id = intval( $_POST["oasiswf"] );
      $reassign_user_id = intval( $_POST["reassign_id"] );
      $reassign_comments = sanitize_text_field( $_POST['reassignComments'] );

      $action_table = OW_Utility::instance()->get_action_table_name();
      $action_history_table = OW_Utility::instance()->get_action_history_table_name();

      $reassign_comments_json = "";
      $reassign_comments_json_array = "";
      if ( !empty( $reassign_comments ) ) {
         $reassign_comments_json = json_encode( array( 'send_id' => $current_user, 'comment' => stripcslashes( $reassign_comments ), "comment_timestamp" => current_time( "mysql" ) ) );
         $reassign_comments_json_array = json_encode( array( array( 'send_id' => $current_user, 'comment' => stripcslashes( $reassign_comments ), "comment_timestamp" => current_time( "mysql" ) ) ) );
      }

      $ow_email = new OW_Email( );
      if ( $action_history_id && $reassign_user_id ) {
         $ow_history_service = new OW_History_Service();
         $action = $ow_history_service->get_action_history_by_id( $action_history_id );

         // insert record into history table regarding this action
         $data = (array) $action;
         if ( $data["assign_actor_id"] != -1 ) { // assignment or publish step (reassigned)
            unset( $data["ID"] );
            if ( empty( $data['due_date'] ) || $data['due_date'] == '0000-00-00' ) {
               unset( $data['due_date'] );
            }
            if ( empty( $data['reminder_date'] ) || $data['reminder_date'] == '0000-00-00' ) {
               unset( $data['reminder_date'] );
            }
            if ( empty( $data['reminder_date_after'] ) || $data['reminder_date_after'] == '0000-00-00' ) {
               unset( $data['reminder_date_after'] );
            }
            $data["assign_actor_id"] = $reassign_user_id;
            $data["from_id"] = $action_history_id;
            $data["create_datetime"] = current_time( 'mysql' );
            if ( !empty( $reassign_comments ) ) {
               $data['comment'] = $reassign_comments_json_array;
            }
            $new_history_id = OW_Utility::instance()->insert_to_table( $action_history_table, $data );
            if ( $new_history_id ) {
               $wpdb->update( $action_history_table, array( "action_status" => "reassigned" ), array( "ID" => $action_history_id, "assign_actor_id" => $current_user ) );
               $ow_email->delete_step_email( $action_history_id, $current_user );
               $ow_email->send_step_email( $new_history_id, $reassign_user_id ); // send mail to the actor .
               echo trim( $new_history_id );
            }
         } else { // review step (reassigned)
            $reviews = $ow_history_service->get_review_action_by_status( "assignment", $action_history_id );
            foreach ( $reviews as $review ) {
               if ( $review->actor_id == $reassign_user_id ) {
                  echo __( "Selected user is already assigned the task. Please select another user.", "oasisworkflow" );
                  exit();
               }
            }
            $review = $ow_history_service->get_review_action_by_actor( $current_user, "assignment", $action_history_id );

            // if the reassign is in a review process, insert data into the fc_action table
            $review = (array) $review;
            $review_id = $review["ID"];
            unset( $review["ID"] );
            if ( empty( $review['due_date'] ) || $review['due_date'] == '0000-00-00' ) {
               unset( $review['due_date'] );
            }
            if ( empty( $review['comments'] ) ) {
               unset( $review['comments'] );
            }
            $review["actor_id"] = $reassign_user_id;

            $new_review_history_id = OW_Utility::instance()->insert_to_table( $action_table, $review );
            if ( $new_review_history_id ) {
               $wpdb->update( $action_table, array( "review_status" => "reassigned", "comments" => $reassign_comments_json_array, "update_datetime" => current_time( "mysql" ) ), array( "ID" => $review_id ) );
               $data = array( "to_id" => $new_review_history_id, "sign_off_date" => current_time( "mysql" ) );
               $ow_email->delete_step_email( $action_history_id, $current_user );
               $ow_email->send_step_email( $action_history_id, $reassign_user_id ); // send mail to the actor .
               echo trim( $new_review_history_id );
            }
         }
         die();
      }
   }

   /*
    * get all the posts in all the workflow
    *
    * @return mixed array of all the post with post_id and post_title
    *
    * @since 2.0
    */

   public function get_posts_in_all_workflow() {
      global $wpdb;
      $sql = "SELECT DISTINCT(A.post_id) as post_id , B.post_title as title
				  	FROM " . OW_Utility::instance()->get_action_history_table_name() . " AS A
					LEFT JOIN
					{$wpdb->posts} AS B
					ON  A.post_id = B.ID
					GROUP BY B.post_title";

      $result = $wpdb->get_results( $sql );
      return $result;
   }

   /*
    * from the given action history, return the sign off date
    * @param mixed $action_history_row - action history row
    * @return string sign off date
    *
    * @since 2.0
    */

   public function get_sign_off_date( $action_history_row ) {
      if ( $action_history_row->action_status == "complete" || $action_history_row->action_status == "submitted" ||
              $action_history_row->action_status == "aborted" || $action_history_row->action_status == "abort_no_action" ) {
         return $action_history_row->create_datetime;
      }

      if ( $action_history_row->action_status == "claim_cancel" ) {
         $ow_history_service = new OW_History_Service();
         $claimed_row = $ow_history_service->get_action_history_by_parameters( "claimed", $action_history_row->step_id, $action_history_row->post_id, $action_history_row->from_id );
         return $claimed_row->create_datetime;
      }

      $ow_history_service = new OW_History_Service();
      $action = $ow_history_service->get_action_history_by_from_id( $action_history_row->ID );
      if ( $action ) {
         return $action->create_datetime;
      }
   }

   /*
    * from the given action history, return the sign off status
    * @param mixed $action_history_row - action history row
    * @return string sign off status
    *
    * @since 2.0
    */

   public function get_sign_off_status( $action_history_row ) {
      if ( $action_history_row->action_status == "submitted" )
         return __( "Submitted", "oasisworkflow" );
      if ( $action_history_row->action_status == "aborted" )
         return __( "Aborted", "oasisworkflow" );
      if ( $action_history_row->action_status == "abort_no_action" )
         return __( "No Action Taken", "oasisworkflow" );
      if ( $action_history_row->action_status == "claim_cancel" )
         return __( "Unclaimed", "oasisworkflow" );
      if ( $action_history_row->action_status == "claimed" )
         return __( "Claimed", "oasisworkflow" );
      if ( $action_history_row->action_status == "reassigned" )
         return __( "Reassigned", "oasisworkflow" );

      // from the next history record determine the status of the workflow
      $ow_history_service = new OW_History_Service();
      $next_history_record = $ow_history_service->get_action_history_by_from_id( $action_history_row->ID );
      if ( !$next_history_record )
         return ""; // this is the latest step, so this step is not yet completed.

      if ( $next_history_record->action_status == "complete" )
         return __( "Workflow completed", "oasisworkflow" );
      if ( $next_history_record->action_status == "cancelled" )
         return __( "Cancelled", "oasisworkflow" );
      $step_info = json_decode( $action_history_row->step_info );
      $process = "";
      if ( !empty( $step_info ) ) {
         $process = $step_info->process;
      }

      $workflow_service = new OW_Workflow_Service();
      $from_step = $action_history_row->step_id;
      $to_step = $next_history_record->step_id;
      $process_outcome = $workflow_service->get_process_outcome( $from_step, $to_step );

      if ( $process == "review" ) {
         if ( $process_outcome == "success" )
            return __( "Approved", "oasisworkflow" );
         if ( $process_outcome == "failure" )
            return __( "Rejected", "oasisworkflow" );
      }
      if ( $process_outcome == "success" )
         return __( "Completed", "oasisworkflow" );
      if ( $process_outcome == "failure" )
         return __( "Unable to Complete", "oasisworkflow" );
   }

   /*
    * from the given review action history, return the sign off status
    * @param mixed $action_history_row - action history row
    * @return string sign off status
    *
    * @since 2.0
    */

   public function get_review_sign_off_status( $action_history_row, $review_row ) {
      if ( $review_row->review_status == "reassigned" )
         return __( "Reassigned", "oasisworkflow" );

      $from_step = $action_history_row->step_id;
      $to_step = $review_row->step_id;
      if ( !($from_step && $to_step ) )
         return "";
      $step_info = json_decode( $action_history_row->step_info );
      $process = "";
      if ( !empty( $step_info ) ) {
         $process = $step_info->process;
      }

      $workflow_service = new OW_Workflow_Service();
      $process_outcome = $workflow_service->get_process_outcome( $from_step, $to_step );

      if ( $process == "review" ) {
         if ( $process_outcome == "success" )
            return __( "Approved", "oasisworkflow" );
         if ( $process_outcome == "failure" )
            return __( "Rejected", "oasisworkflow" );
      }
      if ( $process_outcome == "success" )
         return __( "Complete", "oasisworkflow" );
      if ( $process_outcome == "failure" )
         return __( "Unable to Complete", "oasisworkflow" );
   }

   /*
    * from the given action history, return the sign off status for the next history record
    * @param mixed $action_history_row - action history row
    * @return string sign off status
    *
    * @since 2.0
    */

   public function get_next_step_sign_off_status( $action_history_row ) {

      $ow_history_service = new OW_History_Service();
      $next_history_object = $ow_history_service->get_action_history_by_from_id( $action_history_row->ID );
      if ( !$next_history_object ) {
         return "";
      } else {
         return $next_history_object->action_status;
      }
   }

   /*
    * for a given history row (sign off step), get the comments count
    *
    * @param mixed $action_history_row - action history row
    * @return int count of comments
    *
    * @since 2.0
    */

   public function get_sign_off_comment_count( $action_history_row ) {
      if ( $action_history_row->action_status == "claimed" ||
              $action_history_row->action_status == "claim_cancel" ||
              $action_history_row->action_status == "complete" ) {
         return "0";
      }
      $ow_history_service = new OW_History_Service();
      $next_history_object = $ow_history_service->get_action_history_by_from_id( $action_history_row->ID );
      if ( is_object( $next_history_object ) ) {
         return $this->get_comment_count( $next_history_object->ID );
      } else
         return 0; // no comments found
   }

   /*
    * for a given review history row (sign off step), get the comments count
    *
    * @param mixed $review_row - review action history row
    * @return int count of comments
    *
    * @since 2.0
    */

   public function get_review_sign_off_comment_count( $review_row ) {
      $i = 0;

      if ( $review_row ) {
         $comments = json_decode( $review_row->comments );
         if ( $comments ) {
            foreach ( $comments as $comment ) {
               if ( $comment->comment )
                  $i ++;
            }
         }
      }
      return $i;
   }

   /*
    * Get all comments related to specific  posts or users
    *
    * @param ID $action_id (could be review action id OR history action id)
    * @param string $page_action history or inbox or something else
    * @return mixed comments array
    * @since 2.0
    */

   public function get_sign_off_comments( $action_id, $page_action = "" ) {
      $action_id = intval( $action_id );

      $ow_history_service = new OW_History_Service();
      $action = $ow_history_service->get_action_history_by_id( $action_id );
      $comments = array();
      $content = "";

      if ( $action && $action->comment != "" ) {
         if ( $action->action_status != "claimed" &&
                 $action->action_status != "claim_cancel" ) {// no comments needed for claimed and claim cancel actions
            $comments = json_decode( $action->comment );
         }
      }

      if ( $action && $action->create_datetime != "" ) {
         $sign_off_date = $action->create_datetime;
      } else {
         $sign_off_date = "";
      }

      $review_rows = $ow_history_service->get_review_action_by_history_id( $action_id, "update_datetime" );
      if ( $review_rows ) {
         foreach ( $review_rows as $review_row ) {
            if ( $review_row->review_status == 'reassigned' ) {
               if ( !empty( $review_row->comments ) ) {
                  $comments = json_decode( $review_row->comments ); // get the latest comment
                  break;
               }
            }
         }
      }

      if ( $page_action != "" && $page_action == "history" ) {
         $action = $ow_history_service->get_action_history_by_from_id( $action_id );
         if ( $action ) {
            if ( $action->comment != "" ) {
               if ( $action->action_status != "claimed" &&
                       $action->action_status != "claim_cancel" ) {// no comments needed for claimed and claim cancel actions
                  $comments = json_decode( $action->comment );
               }
            }
         }
      }

      if ( $page_action != "" && $page_action == "review" ) {
         $sign_off_date = "";
         $action = $ow_history_service->get_review_action_by_id( $action_id );
         if ( $action ) {
            $comments = json_decode( $action->comments );
            $sign_off_date = $action->update_datetime;
         }
      }

      foreach ( $comments as $key => $comment ) {
         if ( $comment->send_id == "System" ) {
            $lbl = "System";
         } else {
            $lbl = OW_Utility::instance()->get_user_name( $comment->send_id );
         }

         //return only comments exclude user and date
         if ( $key >= 0 ) {
            $content .= nl2br( $comment->comment );
         } else {
            $content .= nl2br( $comment->comment ) . "\t";
         }
      }
      return $content;
   }

   /**
    * Get a map of action history and related comments, action_history_id is the key while comments is the value.
    * comments can be an array too.
    */
   public function get_sign_off_comments_map( $post_id ) {
   	global $wpdb;
   	$post_id = intval( $post_id );

   	$comments_map = array();
   	$action_history_ids = array();

   	// get the comments for the post from the action_history
   	$results = $this->get_assignment_comment_for_post( $post_id );
   	if ( $results ) {
   		foreach ( $results as $result ) {
   			$action_history_ids[] = $result->ID;
   			$comments = "";
   			if ( ! empty( $result->comment ) ) {
   				$comments = json_decode( $result->comment );
   			}

   			$comments_map[$result->ID] = $comments;
   		}
   	}

   	// in case of reassign within the review action, the comments are stored in the action table.
   	if ( ! empty( $action_history_ids ) ) {
   		$results = $this->get_reassigned_comments_for_review_steps( $action_history_ids );
   		if ( $results ) {
   			foreach ( $results as $result ) {
   				$comments = "";
   				if ( !empty( $result->comments ) ) {
   					$comments = json_decode( $result->comments );
   				}

   				// add to the existing array
   				if ( array_key_exists( $result->action_history_id, $comments_map ) ) {
   					array_push( $comments_map[$result->action_history_id], $comments[0] );
   				}
   			}
   		}
   	}

   	return $comments_map;

   }

   /*
    * See whether it's a submit to workflow
    *
    * @since 2.0
    */

   public function workflow_submit_check( $selected_user ) {

      $page_var = isset( $_GET['page'] ) ? sanitize_text_field( $_GET["page"] ) : "";
      $selected_user = intval( sanitize_text_field( $selected_user ) );
      $post_id = "";
      if ( isset( $_GET['post'] ) ) {
         $post_id = intval( sanitize_text_field( $_GET["post"] ) );
      }

      $ow_history_service = new OW_History_Service();

      //inbox
      $page_var = isset( $_GET['page'] ) ? sanitize_text_field( $_GET["page"] ) : "";
      if ( $page_var == 'oasiswf-inbox' ) {
         return "inbox";
      }

      // submit to workflow
      if ( is_array( $post_id ) ) {//looks like the user is performing a bulk action, and hence we need not load the workflow javascripts
         return false;
      }
      $current_tasks = $ow_history_service->get_action_history_by_status( "assignment", $post_id );
      if ( count( $current_tasks ) == 0 ) {
         return "submit";
      }

      // sign off
      if ( !empty( $post_id ) && isset( $_GET['action'] ) && sanitize_text_field( $_GET["action"] ) == "edit" ) {

         $row = $this->get_assigned_post( $post_id, $selected_user, "row" );
         if ( $row ) {
            return $row->ID;
         }
      }
      return false;
   }

   /*
    * Internal function - to get users in step
    * @param int $step_id
    * @param int|null $post_id
    * @return mixed users and processes in the step
    */

   public function get_users_in_step_internal( $step_id, $post_id = null ) {
      if ( $step_id == "nodefine" ) {
         return null;
      }

      $workflow_service = new OW_Workflow_Service();

      $result = null;
      $wf_info = $workflow_service->get_step_by_id( $step_id );
      if ( $wf_info ) {
         $step_info = json_decode( $wf_info->step_info );
         $users = OW_Utility::instance()->get_users_by_role( $step_info->assignee, $post_id );
         if ( $users ) {
            $result["users"] = $users;
            $result["process"] = $step_info->process;
            $result["assign_to_all"] = isset( $step_info->assign_to_all ) ? $step_info->assign_to_all : 0;
         }
      }
      return $result;
   }

   /*
    * Get users in step - ajax function
    *
    * @return mixed users and processes in the step
    */

   public function get_users_in_step() {
      check_ajax_referer( 'owf_signoff_ajax_nonce', 'security' );

      $step_id = intval( $_POST["stepid"] );
      $post_id = isset( $_POST["postid"] ) ? intval( sanitize_text_field( $_POST["postid"] ) ) : null;

      $users_and_processes = $this->get_users_in_step_internal( $step_id, $post_id );
      if ( $users_and_processes != null ) {
         echo json_encode( $users_and_processes );
      } else {
         echo "no users found";
      }

      die();
   }

   /*
    * Internal function to get the next or previous steps
    * @param int $action_history_id
    * @return mixed step information
    *
    * @since 2.0
    */

   public function get_pre_next_steps_internal( $action_history_id ) {
      $action_history_id = intval( sanitize_text_field( $action_history_id ) );

      $ow_history_service = new OW_History_Service();
      $ow_workflow_service = new OW_Workflow_Service();
      $action_history = $ow_history_service->get_action_history_by_id( $action_history_id );
      $steps = $ow_workflow_service->get_process_steps( $action_history->step_id );
      return $steps;
   }

   /*
    * function to get the next or previous steps - ajax function
    * @param int $action_history_id
    * @return mixed step information
    *
    * @since 2.0
    */

   public function get_pre_next_steps() {
      check_ajax_referer( 'owf_signoff_ajax_nonce', 'security' );

      $action_history_id = intval( $_POST["oasiswfId"] );
      $steps = $this->get_pre_next_steps_internal( $action_history_id );

      echo json_encode( $steps );

      die();
   }

   /*
    * when the workflow progresses to the next step, we need to update the post status with the status set in the step
    * @param int $post_id
    * @param int $step_id
    * @param string $step_outcome - complete or not
    * @param string $immediately - do you want to publish the article immediately or at a set time.
    */

   public function copy_step_status_to_post( $post_id, $step_id, $step_outcome, $immediately = null ) {
      global $wpdb;

      $step_id = intval( $step_id );
      $post_id = intval( $post_id );
      $step_outcome = sanitize_text_field( $step_outcome );
      if ( !empty( $immediately ) ) {
         $immediately = sanitize_text_field( $immediately );
      }

      $ow_workflow_service = new OW_Workflow_Service();
      $step = $ow_workflow_service->get_step_by_id( $step_id );
      if ( $step ) {
         $step_info = json_decode( $step->step_info );

         if ( $step_outcome == "complete" )
            $step_status = $step_info->status;
         else
            $step_status = $step_info->failure_status;

         if ( $step_status ) {
            if ( $immediately ) {
               $im_dt = new DateTime( $immediately );
               $now_dt = new DateTime( current_time( 'mysql' ) );
               $original_post_id = get_post_meta( $post_id, 'oasis_original', true );
               if ( $step_status == "publish" && $im_dt > $now_dt ) {
                  if ( empty( $original_post_id ) ) {
                     $step_status = "future";
                  } else {
                     $step_status = "owf_scheduledrev";
                  }
               }

               /**
                * added parameter to wp_update_post arguments
                * For more details visit below link:
                * http://codex.wordpress.org/Function_Reference/wp_update_post#Scheduling_posts
                *
                * @param 'edit_date=>true'
                */
               $publish_post = array(
                   "ID" => $post_id,
                   "post_date_gmt" => get_gmt_from_date( date( "Y-m-d H:i:s", strtotime( $immediately ) ) ),
                   "post_date" => date( "Y-m-d H:i:s", strtotime( $immediately ) ),
                   "post_status" => $step_status,
                   "edit_date" => true
               );
               remove_action( 'save_post', array( $this, 'update_oasis_is_in_workflow' ), 10 );
               wp_update_post( $publish_post );
               add_action( 'save_post', array( $this, 'update_oasis_is_in_workflow' ), 10, 1 );
            } else {
               $update_post = array(
                   "ID" => $post_id,
                   "post_status" => $step_status
               );
               remove_action( 'save_post', array( $this, 'update_oasis_is_in_workflow' ), 10 );
               wp_update_post( $update_post );
               add_action( 'save_post', array( $this, 'update_oasis_is_in_workflow' ), 10, 1 );
            }
         }
      }
   }

   /*
    * called after the step or workflow is completed
    */

   public function update_oasis_is_in_workflow( $post_id ) {
      global $wpdb;

      $post_id = intval( $post_id );
      $is_in_workflow = get_post_meta( $post_id, 'oasis_is_in_workflow', true );

      $ow_history_service = new OW_History_Service();

      // if in workflow and all assignments are completed then call cleanup_after_workflow_complete function
      if ( !empty( $is_in_workflow ) && $is_in_workflow == 1 ) {
         $workflow_assignment_results = $wpdb->get_results( $wpdb->prepare( "SELECT A.*, B.review_status, B.actor_id, B.next_assign_actors, B.step_id as review_step_id, B.action_history_id, B.update_datetime FROM
							(SELECT * FROM " . OW_Utility::instance()->get_action_history_table_name() . " WHERE action_status = 'assignment') as A
							LEFT OUTER JOIN
							(SELECT * FROM " . OW_Utility::instance()->get_action_table_name() . " WHERE review_status = 'assignment') as B
							ON A.ID = B.action_history_id WHERE post_id = %d", $post_id ) );

         if ( count( $workflow_assignment_results ) == 0 ) {
            $this->cleanup_after_workflow_complete( $post_id );
         }

         $email_settings = get_option( 'oasiswf_email_settings' );
         $ow_email = new OW_Email( );
         if ( isset( $email_settings['unauthorized_post_update_emails'] ) && $email_settings['unauthorized_post_update_emails'] == "yes" && count( $workflow_assignment_results ) > 0 ) {
            $create_datetime = new DateTime( $workflow_assignment_results[0]->create_datetime );
            $current_datetime = new DateTime( current_time( 'mysql' ) );
            $diff = $current_datetime->diff( $create_datetime );
            // essentially this method will be called after the user has signed off, so the best way to check if this was not part of sign off is to find the time elapsed
            // more than 2 minutes have passed
            $assignee_arr = array();

            if ( $diff->h > 0 || $diff->i > 1 ) {
               foreach ( $workflow_assignment_results as $assignment ) {
                  $history_details = $ow_history_service->get_action_history_by_id( $assignment->ID );
                  if ( $history_details->assign_actor_id == -1 ) { // review process
                     $action_details = $ow_history_service->get_review_action_by_history_id( $assignment->ID );
                     foreach ( $action_details as $action ) {
                        if ( !in_array( $action->actor_id, $assignee_arr ) ) {
                           array_push( $assignee_arr, $action->actor_id );
                        }
                     }
                  } else { //assignment or publish process
                     if ( !in_array( $history_details->assign_actor_id, $assignee_arr ) ) {
                        array_push( $assignee_arr, $history_details->assign_actor_id );
                     }
                  }
               }
            }
         }
      }
   }

   // Add Oasis Workflow sign off buttons on the edit post link, if the item is in workflow
   public function oasis_edit_post_link( $url, $post_id, $context ) {
      global $wpdb;
      $new_url = $url;

      // lets check given post_id is in workflow
      $row_id = (int) $wpdb->get_var(
              $wpdb->prepare( "SELECT AH.ID FROM " . OW_Utility::instance()->get_action_history_table_name() ." AH
                                 LEFT OUTER JOIN " . OW_Utility::instance()->get_action_table_name() ." A ON AH.ID = A.action_history_id
                                 AND A.review_status = 'assignment'
                                 WHERE AH.action_status = 'assignment'
                                 AND (AH.assign_actor_id = %d OR A.actor_id = %d )
                                 AND post_id = %d LIMIT 0, 1",
                      get_current_user_id(), get_current_user_id(), $post_id ) );

      if ( $row_id && $row_id > 0 ) {
         $new_url = $url . '&oasiswf=' . $row_id;
      }
      return $new_url;
   }

   public function ootb_publish_section_hide() {
      echo "<script type='text/javascript'>
					jQuery(document).ready(function() {
						jQuery('#publish, .misc-pub-section-last').hide() ;
						jQuery('#misc-publishing-actions').children('.curtime').hide() ;
						jQuery('#post-status-display').parent().hide() ;
					});
				</script>";
   }

   /*
    * clean up the options after workflow is completed
    */

   public function cleanup_after_workflow_complete( $post_id ) {
      $post_id = intval( $post_id );
      update_post_meta( $post_id, "oasis_is_in_workflow", 0 ); // set the post meta to 0, specifying that the post is out of a workflow.
   }

   /*
    * this function will simply insert the data for the next step and update the previous action as "processed"
    */

   public function save_action( $data, $actors, $action_id = null ) {
      // reminder days BEFORE the due date
      $reminder_days = get_option( "oasiswf_reminder_days" );
      if ( $reminder_days && isset( $data["due_date"] ) ) {
         $data["reminder_date"] = OW_Utility::instance()->get_pre_next_date( $data["due_date"], "pre", $reminder_days );
      }

      // reminder days AFTER the due date
      $reminder_days_after = get_option( "oasiswf_reminder_days_after" );
      if ( $reminder_days_after && isset( $data["due_date"] ) ) {
         $data["reminder_date_after"] = OW_Utility::instance()->get_pre_next_date( $data["due_date"], "next", $reminder_days_after );
      }

      $ow_workflow_service = new OW_Workflow_Service();
      $action_history_table = OW_Utility::instance()->get_action_history_table_name();
      $action_table = OW_Utility::instance()->get_action_table_name();
      $wf_info = $ow_workflow_service->get_step_by_id( $data["step_id"] );
      if ( $wf_info ) {
         $step_info = json_decode( $wf_info->step_info );
      }

      $ow_email = new OW_Email( );

      if ( $step_info->process == "assignment" || $step_info->process == "publish" ) { //multiple actors are assigned in assignment/publish step
         if ( is_numeric( $actors ) ) {
            $arr[] = $actors;
         } else {
            $arr = explode( "@", $actors );
         }

         for ( $i = 0; $i < count( $arr ); $i ++ ) {
            $data["assign_actor_id"] = $arr[$i];
            $new_action_history_id = OW_Utility::instance()->insert_to_table( $action_history_table, $data );
            do_action( 'owf_save_workflow_signoff_action', $data["post_id"], $new_action_history_id );
            $ow_email->send_step_email( $new_action_history_id ); // send mail to the actor .
         }
      } else if ( $step_info->process == "review" ) {
         $data["assign_actor_id"] = -1;
         $new_action_history_id = OW_Utility::instance()->insert_to_table( $action_history_table, $data );
         do_action( 'owf_save_workflow_signoff_action', $data["post_id"], $new_action_history_id );

         $review_data = array(
             'review_status' => 'assignment',
             'action_history_id' => $new_action_history_id
         );

         if ( is_numeric( $actors ) ) {
            $arr[] = $actors;
         } else {
            $arr = explode( "@", $actors );
         }

         for ( $i = 0; $i < count( $arr ); $i ++ ) {
            if ( !$arr[$i] )
               continue;
            $review_data["actor_id"] = $arr[$i];
            OW_Utility::instance()->insert_to_table( $action_table, $review_data );
            $ow_email->send_step_email( $new_action_history_id, $arr[$i] ); // send mail to the actor .
         }
      }

      //some clean up, only if there is a previous history about the action
      if ( $action_id ) {
         global $wpdb;
         $wpdb->update( $action_history_table, array( "action_status" => "processed" ), array( "ID" => $action_id ) );
         // delete all the unsend emails for this workflow
         $ow_email->delete_step_email( $action_id );
      }

      return $new_action_history_id;
   }

   /*
    * submit post to workflow - internal
    */

   public function submit_post_to_workflow_internal( $step_id, $post_id, $actors, $due_date, $user_comments ) {

      $step_id = intval( $step_id );
      $post_id = intval( $post_id );
      if ( !empty( $due_date ) ) {
         $due_date = sanitize_text_field( $due_date );
      }
      $actors = sanitize_text_field( $actors );
      $user_id = get_current_user_id();

      $ow_workflow_service = new OW_Workflow_Service();

      $comments[] = array( "send_id" => $user_id, "comment" => stripcslashes( $user_comments ), "comment_timestamp" => current_time( "mysql" ) );
      $comments_json = json_encode( $comments );

      //--- post create and sign off by admin ----
      $post = get_post( $post_id );
      $user = OW_Utility::instance()->get_user_name( $user_id );
      $acomments[] = array( "send_id" => "System", "comment" => "Post/Page was submitted to the workflow by " . $user, "comment_timestamp" => current_time( "mysql" ) );
      $adata = array(
          'action_status' => "submitted",
          'comment' => json_encode( $acomments ),
          'step_id' => $step_id,
          'post_id' => $post_id,
          'from_id' => '0',
          'create_datetime' => $post->post_date
      );
      $action_history_table = OW_Utility::instance()->get_action_history_table_name();
      $adata["assign_actor_id"] = $user_id;
      $new_action_history_id = OW_Utility::instance()->insert_to_table( $action_history_table, $adata );  // insert record in history table for workflow submit
      //-----------------------------------------
      $data = array(
          'action_status' => "assignment",
          'comment' => $comments_json,
          'step_id' => $step_id,
          'post_id' => $post_id,
          'from_id' => $new_action_history_id,
          'create_datetime' => current_time( 'mysql' )
      );
      if ( !empty( $due_date ) ) {
         $data["due_date"] = OW_Utility::instance()->format_date_for_db_wp_default( $due_date );
      }
      $new_action_history_id = $this->save_action( $data, $actors );
      update_post_meta( $post_id, "oasis_is_in_workflow", 1 ); // set the post meta to 1, specifying that the post is in a workflow.

      do_action( 'owf_submit_to_workflow', $post_id, $new_action_history_id );
   }


   /**
    * A pre hook before submit to workflow. Calls - owf_submit_to_workflow_pre hook.
    *
    * Can be used by add-ons or plugins to insert any functionality before submit to workflow.
    *
    * @return $result - could be an array OR "true"
    * @since 2.0
    */
   public function submit_post_to_workflow_pre() {

      if ( ! current_user_can( 'edit_posts' ) && ! current_user_can( 'edit_pages' ) ) {
         wp_die( __( 'You are not allowed to create/edit post.' ) );
      }

   	check_ajax_referer( 'owf_signoff_ajax_nonce', 'security' );
		$security_nonce = $_POST['security'];
   	// sanitize the data
   	$step_id = intval( sanitize_text_field( $_POST["step_id"] ) );

      $form = $_POST['form'];
      parse_str( $form, $_POST );

      $post_tags = 0;
      if( ! empty( $_POST['tax_input']['post_tag'] ) ) {
         $post_tags = count( explode( ',', sanitize_text_field( $_POST['tax_input']['post_tag'] ) ) );
      }
      // for some reason, there is an entry for "0" in the list, so lets minus that
      $post_category = (isset($_POST['post_category'])) ? intval( count( $_POST['post_category'] ) ) - 1 : 0;

      // TODO: there is a function to check $_POST['index'] is array or not in wordpress
      $data = @array_map( 'esc_attr', $_POST );
      $post_id = $data['post_ID'];

      // returns the post id of autosave post
      // ref :  https://wordpress.org/support/topic/need-to-invoke-autosave-programmatically-before-running-a-custom-action?replies=4#post-7853159
      //$saved_post_id = wp_create_post_autosave( $_POST );
   	$result = array();
      $post_data = array();
      $post_data['post_title'] = $data['post_title'];
      $post_data['post_content'] = $_POST['content'];
      $post_data['post_excerpt'] = isset($data['excerpt']) ? $data['excerpt'] : '';
      $post_data['post_tag'] = $post_tags;
      $post_data['category'] = $post_category;

   	$args = array( $post_id, $step_id, $result, $post_data );

   	// let the hook run some pre validations and return validation error messages
   	do_action_ref_array( 'owf_submit_to_workflow_pre', array( &$args ) );
   	$result = $args[2];

   	if ( !empty ($result ) ) {
   		echo $result;
   	} else {
   		echo "true"; // echo true if there are no results to be displayed
   	}

   	wp_die();
   }

   /**
    * A pre hook before submit to workflow. Calls - owf_sign_off_from_workflow_pre hook.
    *
    * Can be used by add-ons or plugins to insert any functionality before submit to workflow.
    *
    * @return $result - could be an array OR "true"
    * @since 2.0
    */
   public function sign_off_post_from_workflow_pre() {

      if ( ! current_user_can( 'edit_posts' ) && ! current_user_can( 'edit_pages' ) ) {
         wp_die( __( 'You are not allowed to create/edit post.' ) );
      }

   	check_ajax_referer( 'owf_signoff_ajax_nonce', 'security' );

   	$step_id = intval( sanitize_text_field( $_POST["step_id"] ) );
      $post_id = intval( sanitize_text_field( $_POST["post_id"] ) );

      // if form attr is set then its value from edit post else from inbox page
      if(isset($_POST['form']) && !empty($_POST['form'])) {
      	$form = $_POST['form'];
      	parse_str( $form, $_POST );

         // for some reason, there is an entry for "0" in the list, so lets minus that
         $post_category = $_POST['post_category'];
         $post_category = intval( count( $_POST['post_category'] ) - 1 );
         $post_tags = 0;
         if( ! empty( $_POST['tax_input']['post_tag'] ) ) {
         	$post_tags = count( explode( ',', sanitize_text_field( $_POST['tax_input']['post_tag'] ) ) );
         }

         $data = @array_map( 'esc_attr', $_POST );
         $sanitize_excerpt = $data['excerpt'];
         $post_contents = $_POST['content'];
      } else {
         $data = (array) get_post( $post_id );
         $post_contents = $data['post_content'];
         $sanitize_excerpt = $data['post_excerpt'];
         $post_tags = count( wp_get_post_tags( $post_id ) );
         $post_category = count( wp_get_post_categories( $post_id ) );
      }

      // returns the post id of autosave post
      //$saved_post_id = wp_create_post_autosave( $_POST );

   	$result = array();
      $post_data = array();
      $post_data['post_title'] = $data['post_title'];
      $post_data['post_content'] = $post_contents;
      $post_data['post_excerpt'] = $sanitize_excerpt;
      $post_data['post_tag'] = $post_tags;
      $post_data['category'] = $post_category;

   	$args = array( $post_id, $step_id, $result, $post_data );

   	// let the hook run some pre validations and return validation error messages
   	do_action_ref_array( 'owf_sign_off_from_workflow_pre', array( &$args ) );
   	$result = $args[2];

   	if ( !empty ($result ) ) {
   		echo $result;
   	} else {
   		echo "true"; // echo true if there are no results to be displayed
   	}

   	wp_die();
   }

   /**
    * Submit post to workflow - ajax step
    */
   public function submit_post_to_workflow() {
      $step_id = intval( $_POST["hi_step_id"] );
      $post_id = intval( $_POST["post_ID"] );
      $actors = sanitize_text_field( $_POST["hi_actor_ids"] );
      $assign_to_all = isset( $_POST['assign_to_all'] ) ? intval( sanitize_text_field( $_POST['assign_to_all'] ) ) : 0;
      if( 1 === $assign_to_all ) {
         $workflow_service = new OW_Workflow_Service();
         $wf_info = $workflow_service->get_step_by_id( $step_id );
         $step_info = json_decode( $wf_info->step_info );
         $users = OW_Utility::instance()->get_users_by_role( $step_info->assignee, $post_id );
         $actors = array();
         foreach( $users as $user ) {
            $actors[] = $user->ID;
         }
         $actors = implode( "@", $actors );
      }
      $actors_info = array( $actors, $step_id, $post_id);
      // by default we get the users assigned to the specified role
      // this action will allow to change the actor list
      do_action_ref_array( 'owf_get_actors', array( &$actors_info ) );
      $actors = $actors_info[0];

      $due_date = sanitize_text_field( $_POST["hi_due_date"] );
      // sanitize_text_field remove line-breaks so do not sanitize it.
      $comments = nl2br( $_POST["hi_comment"] );
      $this->submit_post_to_workflow_internal( $step_id, $post_id, $actors, $due_date, $comments);
   }

   /*
    * submit post to step internal function
    */

   public function submit_post_to_step_internal( $post_id, $history_id, $step_id, $task_actor_id, $sign_off_comments, $assigned_actors, $sign_off_status, $due_date ) {
      global $wpdb;

      $ow_workflow_service = new OW_Workflow_Service();
      $ow_history_service = new OW_History_Service();
      // get the history details from fc_action_history
      $history_details = $ow_history_service->get_action_history_by_id( $history_id );

      // comments added during signoff
      $comments[] = array( "send_id" => $task_actor_id, "comment" => stripcslashes( $sign_off_comments ), "comment_timestamp" => current_time( "mysql" ) );

      $actors = $assigned_actors;

      if ( $history_details->assign_actor_id == -1 ) { // the current step is a review step, so review decision check is required
         // let's first save the review action
         // find the next assign actors
         if ( is_numeric( $actors ) ) {
            $next_assign_actors[] = $actors;
         } else {
            $arr = explode( "@", $actors );
            $next_assign_actors = $arr;
         }

         $review_data = array(
             "review_status" => $sign_off_status,
             "next_assign_actors" => json_encode( $next_assign_actors ),
             "step_id" => $step_id, // represents success/failure step id
             "comments" => json_encode( $comments ),
             "update_datetime" => current_time( 'mysql' )
         );

         if ( !empty( $due_date ) ) {
            $review_data["due_date"] = OW_Utility::instance()->format_date_for_db_wp_default( $due_date );
         }

         $action_table = OW_Utility::instance()->get_action_table_name();
         $wpdb->update( $action_table, $review_data, array( "actor_id" => $task_actor_id, "action_history_id" => $history_id ) );

         // invoke the review step procedure to make a review decision
         $new_action_history_id = $this->review_step_procedure( $history_id );

      } else { // the current step is either an assignment or publish step, so no review decision check required
         $data = array(
             'action_status' => "assignment",
             'comment' => json_encode( $comments ),
             'step_id' => $step_id,
             'post_id' => $post_id,
             'from_id' => $history_id,
             'create_datetime' => current_time( 'mysql' )
         );
         if ( !empty( $due_date ) ) {
            $data["due_date"] = OW_Utility::instance()->format_date_for_db_wp_default( $due_date );
         }

         // insert data from the next step
         $new_action_history_id = $this->save_action( $data, $actors, $history_id );

         //------post status change----------
         $this->copy_step_status_to_post( $post_id, $history_details->step_id, $sign_off_status );
      }

      $action_history = $ow_history_service->get_action_history_by_id( $history_id );
      $from_step_id = $action_history->step_id;
      $to_step_id = $step_id;
      $post_id = $post_id;
      $step = $ow_workflow_service->get_step_by_id( $from_step_id );
      $workflow = $ow_workflow_service->get_workflow_by_id( $step->workflow_id );

      do_action( 'owf_step_sign_off', $post_id, $new_action_history_id );
   }

   /*
    * Submit post to step - ajax function
    */

   public function submit_post_to_step() {
      check_ajax_referer( 'owf_signoff_ajax_nonce', 'security' );

      $post_id = intval( $_POST["post_ID"] );
      $step_id = intval( $_POST["hi_step_id"] );
      $history_id = intval( $_POST["oasiswf"] );

      $assign_to_all = intval($_POST['assign_to_all']);
      if( $assign_to_all == 1) {
         $workflow_service = new OW_Workflow_Service();
         $wf_info = $workflow_service->get_step_by_id( $step_id );
         $step_info = json_decode( $wf_info->step_info );
         $users = OW_Utility::instance()->get_users_by_role( $step_info->assignee, $post_id );
         $assigned_actors = array();
         foreach($users as $user) {
            $assigned_actors[] = $user->ID;
         }
         $assigned_actors = implode("@", $assigned_actors);
      } else {
         $assigned_actors = sanitize_text_field( $_POST["hi_actor_ids"] );
      }

      $task_user = get_current_user_id();
      // find out who is signing off the task; sometimes the admin can signoff on behalf of the actual user
      if ( isset( $_POST["hi_task_user"] ) && $_POST["hi_task_user"] != "" ) {
         $task_user = intval( sanitize_text_field( $_POST["hi_task_user"] ) );
      }

      // sanitize_text_field remove line-breaks so do not sanitize it.
      $sign_off_comments = nl2br( $_POST["hi_comment"] );
      $sign_off_status = sanitize_text_field( $_POST['review_result'] );

      $due_date = "";
      if ( isset( $_POST["hi_due_date"] ) && !empty( $_POST["hi_due_date"] ) ) {
         $due_date = sanitize_text_field( $_POST["hi_due_date"] );
      }

      $this->submit_post_to_step_internal( $post_id, $history_id, $step_id, $task_user, $sign_off_comments, $assigned_actors, $sign_off_status, $due_date );
   }

   /*
    * process for review step
    */

   public function review_step_procedure( $action_history_id ) {

      $action_history_id = intval( $action_history_id );

      // get the review settings
      $review_setting = get_option( 'oasiswf_review_process_setting' );

      $ow_workflow_service = new OW_Workflow_Service();
      $ow_history_service = new OW_History_Service();

      // get the review action details from fc_action for the given history_id
      $total_reviews = $ow_history_service->get_review_action_by_history_id( $action_history_id );

      // create a consolidated view of all the reviews, so far
      if ( $total_reviews ) {
         foreach ( $total_reviews as $review ) {
            $next_assign_actors = json_decode( $review->next_assign_actors );
            if ( empty( $next_assign_actors ) ) { // the action is still not completed by the user
               $r = array(
                   "re_actor_id" => $next_assign_actors,
                   "re_step_id" => $review->step_id,
                   "re_comment" => $review->comments,
                   "re_due_date" => $review->due_date
               );
               $review_data[$review->review_status][] = $r;
            } else { // action completed by user and we know the review results
               foreach ( $next_assign_actors as $actor ) :
                  $r = array(
                      "re_actor_id" => $actor,
                      "re_step_id" => $review->step_id,
                      "re_comment" => $review->comments,
                      "re_due_date" => $review->due_date
                  );
                  $review_data[$review->review_status][] = $r;
               endforeach;
            }
         }
      }

      $new_action_history_id = 0;
      switch ( $review_setting ) {
         case "everyone":
            $new_action_history_id = $this->review_step_everyone( $review_data, $action_history_id );
            break;
         case "anyone":
            $new_action_history_id  = $this->review_step_anyone( $review_data, $action_history_id );
            break;
         case "more_than_50":
            $new_action_history_id = $this->review_step_more_50( $review_data, $action_history_id );
            break;
      }

      return $new_action_history_id;
   }

   /*
    * everyone has to approve before the item moves to the next step
    */

   public function review_step_everyone( $review_data, $action_history_id ) {
      /*
       * If assignment (not yet completed) are found, return false; we cannot make any decision yet
       * If we find even one rejected review, complete the step as failed.
       * If all the reviews are approved, then move to the success step.
       */

      if ( isset( $review_data["assignment"] ) && $review_data["assignment"] )
         return 0; // there are users who haven't completed their review

      if ( isset( $review_data["unable"] ) && $review_data["unable"] ) { // even if we see one rejected, we need to go to failure path.
         $new_action_history_id = $this->save_review_action( $review_data["unable"], $action_history_id, "unable" );
         return $new_action_history_id; // since we found our condition
      }

      if ( isset( $review_data["complete"] ) && $review_data["complete"] ) { // looks like we only have completed/approved reviews, lets complete this step.
         $new_action_history_id = $this->save_review_action( $review_data["complete"], $action_history_id, "complete" );
         return $new_action_history_id; // since we found our condition
      }
   }

   /*
    * anyone has to approve before the item moves to the next step
    */

   public function review_step_anyone( $review_data, $action_history_id ) {

      /*
       * First find any approved review, if found, complete the step as pass.
       * If no approved reviews are found, try to find a rejected review. If found, complete the step as failed.
       * Ignore if there are reviews, still in assignment (not yet completed)
       */

      if ( isset( $review_data["complete"] ) && $review_data["complete"] ) { // looks like at least one has approved, lets complete this step.
         $new_action_history_id = $this->save_review_action( $review_data["complete"], $action_history_id, "complete" );

         // change review status on remaining/not completed tasks as "no_action"
         if ( isset( $review_data["assignment"] ) && $review_data["assignment"] ) {
            $this->change_review_status_to_no_action( $review_data["assignment"], $action_history_id );
         }

         return $new_action_history_id; // since we found our condition
      }

      if ( isset( $review_data["unable"] ) && $review_data["unable"] ) { // looks like at least one has rejected, we need to go to failure path.
         $new_action_history_id = $this->save_review_action( $review_data["unable"], $action_history_id, "unable" );

         // change review status on remaining/not completed tasks as "no_action"
         if ( isset( $review_data["assignment"] ) && $review_data["assignment"] ) {
            $this->change_review_status_to_no_action( $review_data["assignment"], $action_history_id );
         }

         return $new_action_history_id; // since we found our condition
      }
   }

   /*
    * more than 50% should approve
    */

   public function review_step_more_50( $review_data, $action_history_id ) {
      $current_assigned_reviews = 0;
      $current_rejected_reviews = 0;
      $current_approved_reviews = 0;

      // get the review action details from fc_action for the given history_id
      $ow_history_service = new OW_History_Service();
      $total_reviews = $ow_history_service->get_review_action_by_history_id( $action_history_id );
      if ( $total_reviews ) {
         foreach ( $total_reviews as $review ) {
            if ( $review->review_status == 'complete' ) {
               $current_approved_reviews ++;
            }
            if ( $review->review_status == 'unable' ) {
               $current_rejected_reviews ++;
            }
            if ( $review->review_status == 'assignment' ) {
               $current_assigned_reviews ++;
            }
         }
      }

      $total_reviews = $current_assigned_reviews + $current_rejected_reviews + $current_approved_reviews;

      $need = floor( $total_reviews / 2 ) + 1; //more than 50%

      if ( $current_approved_reviews >= $need && isset( $review_data["complete"] ) && $review_data["complete"] ) { // looks like we have more than 50% approved, lets complete this step.
         $new_action_history_id = $this->save_review_action( $review_data["complete"], $action_history_id, "complete" );

         // change review status on remaining/not completed tasks as "no_action"
         if ( isset( $review_data["assignment"] ) && $review_data["assignment"] ) {
            $this->change_review_status_to_no_action( $review_data["assignment"], $action_history_id );
         }

         return $new_action_history_id; // since we found our condition
      }

      if ( $current_rejected_reviews >= $need && isset( $review_data["unable"] ) && $review_data["unable"] ) { // looks like we have more than 50% rejected, we need to go to failure path.
         $new_action_history_id = $this->save_review_action( $review_data["unable"], $action_history_id, "unable" );

         // change review status on remaining/not completed tasks as "no_action"
         if ( isset( $review_data["assignment"] ) && $review_data["assignment"] ) {
            $this->change_review_status_to_no_action( $review_data["assignment"], $action_history_id );
         }

         return $new_action_history_id; // since we found our condition
      }

      /*
       * in case, we have equal number of approved and rejected reviews (2 approved and 2 rejected),
       * and to make a decision we need more than 2 (more than 50%)
       * and we have no more assignments left,
       * we should take the failure path
       */
      if ( $current_rejected_reviews == $current_approved_reviews && !isset( $review_data["assignment"] ) ) {
         $new_action_history_id = $this->save_review_action( $review_data["unable"], $action_history_id, "unable" );
         return $new_action_history_id; // since we found our condition
      }
   }

   /*
    * Change review action to no action for people whose tasks were skipped
    */

   public function change_review_status_to_no_action( $ddata, $action_history_id ) {
      global $wpdb;

      for ( $i = 0; $i < count( $ddata ); $i ++ ) {
         $review_data = array(
             "review_status" => "no_action",
             "update_datetime" => current_time( 'mysql' )
         );

         $action_table = OW_Utility::instance()->get_action_table_name();
         $wpdb->update( $action_table, $review_data, array( "review_status" => "assignment", "action_history_id" => $action_history_id ) );
      }
   }

   /*
    * Save the review action data
    */

   public function save_review_action( $ddata, $action_history_id, $result ) {
      $ow_history_service = new OW_History_Service();
      $action = $ow_history_service->get_action_history_by_id( $action_history_id );

      $review_data = array(
          'action_status' => "assignment",
          'post_id' => $action->post_id,
          'from_id' => $action->ID,
          'create_datetime' => current_time( 'mysql' )
      );

      $next_assign_actors = array();
      $all_comments = array();
      $due_date = '';
      for ( $i = 0; $i < count( $ddata ); $i ++ ) {
         if ( !in_array( $ddata[$i]["re_actor_id"], $next_assign_actors ) ) { //only add unique actors to the array
            $next_assign_actors[] = $ddata[$i]["re_actor_id"];
         }

         // combine all commments into one set
         $temp_comment = json_decode( $ddata[$i]["re_comment"], true );
         foreach ( $temp_comment as $temp_key => $temp_value ) {
            $exists = 0;
            foreach ( $all_comments as $all_key => $all_value ) {
               if ( $all_value["send_id"] === $temp_value["send_id"] ) { // if the comment already exists, then skip it
                  $exists = 1;
               }
            }
            if ( $exists == 0 ) {
               $all_comments[] = $temp_value;
            }
         }
         // TODO: temp fix - it takes the last action assigned step
         $next_step_id = $ddata[$i]["re_step_id"];

         //-----get minimal due date--------
         $temp1_date = OW_Utility::instance()->get_date_int( $ddata[$i]["re_due_date"] );
         if ( !empty( $due_date ) ) {
            $temp2_date = OW_Utility::instance()->get_date_int( $due_date );
            $due_date = ( $temp1_date < $temp2_date ) ? $ddata[$i]["re_due_date"] : $due_date;
         } else {
            $due_date = $ddata[$i]["re_due_date"];
         }
      }

      $next_actors = implode( "@", $next_assign_actors );
      $review_data["comment"] = json_encode( $all_comments );
      if ( !empty( $due_date ) ) {
         $review_data["due_date"] = $due_date;
      }
      $review_data["step_id"] = $next_step_id;

      // we have all the data to generated the next set of tasks

      $new_action_history_id = $this->save_action( $review_data, $next_actors, $action->ID );

      //--------post status change---------------
      $this->copy_step_status_to_post( $action->post_id, $action->step_id, $result );

      return $new_action_history_id;
   }

   /*
    * Internal function to change the workflow status to complete
    */

   public function change_workflow_status_to_complete_internal( $post_id, $publish_datetime, $history_id, $current_actor_id ) {
      global $wpdb;

      $post_id = intval( $post_id );
      $history_id = intval( $history_id );
      $current_actor_id = intval( $current_actor_id );
      if ( $publish_datetime != null ) {
         $publish_datetime = sanitize_text_field( $publish_datetime );
      }

      if ( $publish_datetime != null ) {
         $im_dt = new DateTime( $publish_datetime );
         $now_dt = new DateTime( current_time( 'mysql' ) );
      }

      $ow_history_service = new OW_History_Service();
      $history = $ow_history_service->get_action_history_by_id( $history_id );
      $currentTime = current_time( 'mysql' );
      $currentDate = date( 'Y-m-d' );
      $data = array(
          'action_status' => "complete",
          'step_id' => $history->step_id,
          'assign_actor_id' => get_current_user_id(),
          'post_id' => $post_id,
          'from_id' => $history_id,
          'comment' => "",
          'create_datetime' => $currentTime
      );

      $action_history_table = OW_Utility::instance()->get_action_history_table_name();
      $action_table = OW_Utility::instance()->get_action_table_name();
      $new_action_history_id = OW_Utility::instance()->insert_to_table( $action_history_table, $data );

      // update action table, if review was the last step in the process
      $wpdb->update( $action_table, array( "review_status" => "complete",
          "update_datetime" => current_time( 'mysql' ) ), array( "action_history_id" => $history->ID ) );

      $ow_email = new OW_Email( );

      if ( $new_action_history_id ) {
         global $wpdb;
         // delete all the unsend emails for this workflow
         $ow_email->delete_step_email( $history_id, $current_actor_id );

         $result = $wpdb->update( $action_history_table, array( 'action_status' => 'processed' ), array( 'ID' => $history_id ) );

         $action = $ow_history_service->get_action_history_by_id( $history_id );

         $original_post_id = get_post_meta( $post_id, 'oasis_original', true );
         // if this is the original post - act on it
         if ( $publish_datetime != null && $now_dt != $im_dt ) {
            $this->copy_step_status_to_post( $post_id, $action->step_id, "complete", $publish_datetime );
         } else {
            $this->copy_step_status_to_post( $post_id, $action->step_id, "complete", current_time( 'mysql' ) );
         }
         $this->cleanup_after_workflow_complete( $post_id );
         return "complete_original";
      }
   }

   /*
    * Change status of workflow to complete - ajax function
    */

   public function change_workflow_status_to_complete() {
      check_ajax_referer( 'owf_signoff_ajax_nonce', 'security' );
      $post_id = intval( $_POST['post_id'] );
      $history_id = intval( $_POST['oasiswf_id'] );
      $publish_datetime = null;
      if ( isset( $_POST["immediately"] ) ) {
         $publish_datetime = sanitize_text_field( $_POST["immediately"] );
      } else {
         $pdata = get_post( $post_id );
         $publish_datetime = get_the_date( 'Y-m-d H:i:s', $post_id );
      }

      $task_user = get_current_user_id();
      // find out who is signing off the task; sometimes the admin can signoff on behalf of the actual user
      if ( isset( $_POST["hi_task_user"] ) && $_POST["hi_task_user"] != "" ) {
         $task_user = intval( sanitize_text_field( $_POST["hi_task_user"] ) );
      }

      $post_complete_action = $this->change_workflow_status_to_complete_internal( $post_id, $publish_datetime, $history_id, $task_user );
      echo $post_complete_action;

      $ow_history_service = new OW_History_Service();
      $ow_workflow_service = new OW_Workflow_Service();
      $history = $ow_history_service->get_action_history_by_id( $history_id );
      $step = $ow_workflow_service->get_step_by_id( $history->step_id );
      $workflow = $ow_workflow_service->get_workflow_by_id( $step->workflow_id );
      do_action( 'owf_workflow_complete', $post_id, $workflow->ID );

      die();
   }

   /*
    * Cancel the workflow - ajax function
    */

   public function change_workflow_status_to_cancelled() {

      check_ajax_referer( 'owf_signoff_ajax_nonce', 'security' );
      $user_id = get_current_user_id();
      $post_id = intval( $_POST["post_id"] );
      $history_id = intval( $_POST["oasiswf_id"] );
      $review_result = sanitize_text_field( $_POST["review_result"] );
      $user_comments = sanitize_text_field( $_POST["hi_comment"] );

      $comments[] = array( "send_id" => $user_id, "comment" => stripcslashes( $user_comments ) );
      $comments_json = json_encode( $comments );

      $data = array(
          'action_status' => "cancelled",
          'comment' => $comments_json,
          'post_id' => $post_id,
          'from_id' => $history_id,
          'create_datetime' => current_time( 'mysql' )
      );
      $action_history_table = OW_Utility::instance()->get_action_history_table_name();
      $new_action_history_id = OW_Utility::instance()->insert_to_table( $action_history_table, $data );

      if ( isset( $_POST["hi_task_user"] ) && $_POST["hi_task_user"] != "" ) {
         $current_actor_id = intval( sanitize_text_field( $_POST["hi_task_user"] ) );
      } else {
         $current_actor_id = get_current_user_id();
      }

      $ow_email = new OW_Email( );
      $ow_history_service = new OW_History_Service();

      if ( $new_action_history_id ) {
         global $wpdb;
         // delete all the unsend emails for this workflow
         $ow_email->delete_step_email( $history_id, $current_actor_id );
         $result = $wpdb->update( $action_history_table, array( 'action_status' => 'processed' ), array( 'ID' => $history_id ) );

         $action = $ow_history_service->get_action_history_by_id( $history_id );

         $this->copy_step_status_to_post( $post_id, $action->step_id, $review_result );

         //-----------------email-----------------------
         $current_userid = get_current_user_id();
         $users = $wpdb->get_results( "SELECT users_1.ID, users_1.display_name FROM {$wpdb->base_prefix}users users_1
			INNER JOIN {$wpdb->base_prefix}usermeta usermeta_1 ON ( users_1.ID = usermeta_1.user_id )
			WHERE (usermeta_1.meta_key = '{$wpdb->prefix}capabilities' AND CAST( usermeta_1.meta_value AS CHAR ) LIKE '%administrator%')" );

         $post = get_post( $post_id );
         $post_author = get_userdata( $post->post_author );
         $title = "'{$post->post_title}' was cancelled from the workflow";
         $full_name = OW_Utility::instance()->get_user_name( $current_userid );

			$msg = sprintf( __( '<div>Hello %1$s,</div><p>The post <a href="%2$s" title="%3$s">%3$s</a> has been cancelled from the workflow.</p>', 'oasisworkflow' ),
						$post_author->display_name,
						esc_url( $post->guid ),
						$post->post_title );

// 			$msg .= __( '<p>This means that something was not good. Please call your approver in order to understand what you should edit.</p>', 'oasisworkflow' );
			if ( !empty ( $user_comments ) ) {
				$msg .= "<p><strong>" . __('Additionally,', "oasisworkflow") . "</strong> {$full_name} " . __('added the following comments', "oasisworkflow") . ":</p>" ;
				$msg .= "<p>" . nl2br( $user_comments ) . "</p>" ;
			}

			$msg .= __("<p>Thanks.</p>", "oasisworkflow");

			$message = '<html><head></head><body><div class="email_notification_body">' . $msg . '</div></body></html>';

			$ow_email = new OW_Email( );
         $ow_email->send_mail( $post->post_author, $title, $message );

         //---------------------------------------------
         $this->cleanup_after_workflow_complete( $post_id );
         echo trim( $new_action_history_id );
      }
      die();
   }

   public function get_step_status_by_history_id() {
      check_ajax_referer( 'owf_signoff_ajax_nonce', 'security' );
      $history_id = intval( $_POST["oasiswf"] );
      $step_result = sanitize_text_field( $_POST["review_result"] );
      $step_status = $this->get_step_status_by_history_id_internal( $history_id, $step_result );

      if ( $step_status ) {
         echo trim( $step_status );
         exit();
      }
   }

   public function get_step_status_by_history_id_internal( $history_id, $step_result ) {
      $ow_history_service = new OW_History_Service();
      $ow_workflow_service = new OW_Workflow_Service();
      $action = $ow_history_service->get_action_history_by_id( $history_id );
      $step_id = $action->step_id;
      $step = $ow_workflow_service->get_step_by_id( $step_id );
      $step_status = null;

      if ( $step ) {
         $step_info = json_decode( $step->step_info );
         $step_status = "draft";

         if ( $step_result == "complete" ) {
            $step_status = $step_info->status;
         } else {
            $step_status = $step_info->failure_status;
         }
         if ( $step_status ) {
            return trim( $step_status );
         }
      }
   }

   public function get_step_status_by_step_id() {
      check_ajax_referer( 'owf_signoff_ajax_nonce', 'security' );
      $step_id = intval( $_POST["step_id"] );
      $ow_workflow_service = new OW_Workflow_Service();
      $step = $ow_workflow_service->get_step_by_id( $step_id );
      $step_result = sanitize_text_field( $_POST["review_result"] );

      if ( $step ) {
         $step_info = json_decode( $step->step_info );
         $step_status = "draft";

         if ( $step_result == "complete" )
            $step_status = $step_info->status;
         else
            $step_status = $step_info->failure_status;

         if ( $step_status ) {
            echo trim( $step_status );
            die();
         }
      }
   }

   public function workflow_submit_action( $location, $post_id ) {
      if ( isset( $_POST["save_action"] ) && $_POST["save_action"] == "submit_post_to_workflow" ) {
         $this->submit_post_to_workflow();
      }

      if ( isset( $_POST["hi_process_info"] ) && $_POST["hi_process_info"] ) {
         $this->set_loading_post_status();
      }

      return $location;
   }

   /*
    * abort workflow
    */

   public function exit_post_from_workflow() {
      global $wpdb;

      $nonce = 'owf_inbox_ajax_nonce';

      if ( isset( $_POST['command'] ) && sanitize_text_field( $_POST['command'] ) == 'exit_from_workflow' ) {
         $nonce = 'owf_exit_post_from_workflow_ajax_nonce';
      }
      check_ajax_referer( $nonce, 'security' );

      if ( ! current_user_can( 'ow_abort_workflow' ) ) {
      	wp_die( __( 'You are not allowed to abort the workflow.' ) );
      }


      $history_id = intval( sanitize_text_field( $_POST["exitId"] ) );

      $ow_history_service = new OW_History_Service();
      $action = $ow_history_service->get_action_history_by_id( $history_id );

      $action_history_table = OW_Utility::instance()->get_action_history_table_name();

      $comment[] = array(
          "send_id" => get_current_user_id(),
          "comment" => "Post/Page was aborted from the workflow."
      );
      $data = array(
          "action_status" => "aborted",
          "post_id" => $action->post_id,
          "comment" => json_encode( $comment ),
          "from_id" => $history_id,
          "step_id" => $action->step_id, // since we do not have the step id information for this
          "assign_actor_id" => get_current_user_id(), // since we do not have anyone assigned anymore.
          'create_datetime' => current_time( 'mysql' )
      );
      $action_table = OW_Utility::instance()->get_action_table_name();
      $iid = OW_Utility::instance()->insert_to_table( $action_history_table, $data );
      $ow_email = new OW_Email( );

      if ( $iid ) {
         // find all the history records for the given post id which has the status = "assignment"
         $post_action_histories = $ow_history_service->get_action_history_by_status( "assignment", $action->post_id );
         foreach ( $post_action_histories as $post_action_history ) {
            // delete all the unsend emails for this workflow
            $ow_email->delete_step_email( $post_action_history->ID );
            // update the current assignments to abort_no_action
            $wpdb->update( $action_history_table, array( "action_status" => "abort_no_action", "create_datetime" => current_time( 'mysql' ) ), array( "ID" => $post_action_history->ID ) );
            // change the assignments in the action table to processed
            $wpdb->update( $action_table, array( "review_status" => "abort_no_action", "update_datetime" => current_time( 'mysql' ) ), array( "action_history_id" => $post_action_history->ID ) );
         }
         $this->cleanup_after_workflow_complete( $action->post_id );

         echo trim( $iid );
      }

      die();
   }

   /*
    * multi-abort for workflow - ajax function
    *
    * @since 2.0
    */

   public function multi_abort_from_workflow() {
      global $wpdb;

      if ( ! current_user_can( 'ow_abort_workflow' ) ) {
      	wp_die( __( 'You are not allowed to abort the workflow.' ) );
      }

      $post_ids = (array) $_POST['postids'];

      // sanitize the values
      $post_ids = array_map( 'esc_attr', $post_ids );

      $ow_history_service = new OW_History_Service();
      foreach ( $post_ids as $post_id ) {
         $row = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM " . OW_Utility::instance()->get_action_history_table_name() .
         		" WHERE post_id = %d AND action_status = 'assignment'", $post_id ) );

         $action = $ow_history_service->get_action_history_by_id( $row->ID );
         $comment[] = array(
             "send_id" => get_current_user_id(),
             "comment" => "Post/Page was aborted from the workflow."
         );
         $data = array(
             "action_status" => "aborted",
             "post_id" => $action->post_id,
             "comment" => json_encode( $comment ),
             "from_id" => $row->ID,
             "step_id" => $action->step_id, // since we do not have the step id information for this
             "assign_actor_id" => get_current_user_id(), // since we do not have anyone assigned anymore.
             'create_datetime' => current_time( 'mysql' )
         );

         $ow_email = new OW_Email( );

         $action_history_table = OW_Utility::instance()->get_action_history_table_name();
         $action_table = OW_Utility::instance()->get_action_table_name();
         $iid = OW_Utility::instance()->insert_to_table( $action_history_table, $data );
         if ( $iid ) {
            // find all the history records for the given post id which has the status = "assignment"
            $post_action_histories = $ow_history_service->get_action_history_by_status( "assignment", $post_id );
            foreach ( $post_action_histories as $post_action_history ) {
               // delete all the unsend emails for this workflow
               $ow_email->delete_step_email( $post_action_history->ID );
               // update the current assignments to abort_no_action
               $wpdb->update( $action_history_table, array( "action_status" => "abort_no_action", "create_datetime" => current_time( 'mysql' ) ),
               		array( "ID" => $post_action_history->ID ) );
               // change the assignments in the action table to processed
               $wpdb->update( $action_table, array( "review_status" => "abort_no_action", "update_datetime" => current_time( 'mysql' ) ),
               		array( "action_history_id" => $post_action_history->ID ) );
            }
            $this->cleanup_after_workflow_complete( $action->post_id );
         }
      }
      echo true;
      die();
   }

   public function get_post_publish_date_edit_format() {
      $post_id = isset( $_POST["post_id"] ) ? intval( $_POST["post_id"] ) : null;
      if ( !empty( $post_id ) ) {
         $publish_date = get_the_date( OASISWF_EDIT_DATE_FORMAT . " @ H:i", $post_id );
         echo $publish_date;
      } else {
         echo "";
      }
      die();
   }

   /*
    * Get immediately publish drop down content
    */

   public function get_immediately_content( $post_id, $status, $is_future_date ) {
      if ( $status != "publish" )
         return;
      if ( $is_future_date ) {
         $date = get_the_date( 'Y-n-d', $post_id );
         $date_array = explode( "-", $date );
         $time = get_the_time( 'G-i', $post_id );
         $time_array = explode( "-", $time );
      } else {
         $date_array = explode( "-", current_time( "Y-n-d" ) );
         $time_array = explode( "-", current_time( "H-i" ) );
      }
      $months = array( 1 => "01-Jan", 2 => "02-Feb", 3 => "03-Mar", 4 => "04-Apr", 5 => "05-May", 6 => "06-Jun", 7 => "07-Jul", 8 => "08-Aug", 9 => "09-Sep", 10 => "10-Oct", 11 => "11-Nov", 12 => "12-Dec" );

      echo "<select id='im-mon'>";
      foreach ( $months as $k => $v ) {
         if ( $date_array[1] * 1 == $k )
            echo "<option value={$k} selected>{$v}</option>";
         else
            echo "<option value={$k}>{$v}</option>";
      }
      echo "</select>";
      $im_day = esc_attr( $date_array[2] );
      $im_year = esc_attr( $date_array[0] );
      $im_hh = esc_attr( $time_array[0] );
      $im_mn = esc_attr( $time_array[1] );

      echo "<input type='text' id='im-day' value='{$im_day}' class='immediately margin' size='2' maxlength='2' autocomplete='off'>,
		<input type='text' id='im-year' value='{$im_year}' class='immediately im-year' size='4' maxlength='4' autocomplete='off'> @
		<input type='text' id='im-hh' value='{$im_hh}' class='immediately' size='2' maxlength='2' autocomplete='off'> :
		<input type='text' id='im-mn' value='{$im_mn}' class='immediately' size='2' maxlength='2' autocomplete='off'>";
   }

   public function redirect_after_signoff( $url ) {
      if ( isset( $_POST['hi_oasiswf_redirect'] ) AND $_POST['hi_oasiswf_redirect'] == 'step' ) {
         wp_redirect( admin_url( 'admin.php?page=oasiswf-inbox' ) );
         die();
      }
      return $url;
   }

   /*
    * Setup the sign off popup and enqueue the related scripts
    *
    * @since 2.0
    *
    */

   public function step_signoff_popup_setup() {
      global $wpdb, $chkResult;
      $selected_user = isset( $_GET['user'] ) ? intval( $_GET['user'] ) : get_current_user_id();

      $chkResult = $this->workflow_submit_check( $selected_user );

      $inbox_service = new OW_Inbox_Service();

      if ( get_option( "oasiswf_activate_workflow" ) == "active" &&
              is_admin() && preg_match_all( '/page=oasiswf(.*)|post-new\.(.*)|post\.(.*)/', $_SERVER['REQUEST_URI'], $matches ) ) {

         if ( $chkResult == "inbox" ) {
            $this->enqueue_and_localize_submit_step_script();

            $inbox_service->enqueue_and_localize_script();
         } else if ( current_user_can( 'ow_submit_to_workflow' ) && $chkResult == "submit" &&
                 is_admin() && preg_match_all( '/page=oasiswf(.*)|post-new\.(.*)|post\.(.*)/', $_SERVER['REQUEST_URI'], $matches ) ) {

            // if the post is published or scheduled, we do not want to show the "submit to workflow" button.
            // submit to workflow is only available for new posts
            if ( isset( $_GET['post'] ) && sanitize_text_field( $_GET["post"] ) &&
            		isset( $_GET['action'] ) && sanitize_text_field( $_GET["action"] ) == "edit" ) {
	            $post_status = get_post_status( $_GET["post"]);
	            if ($post_status == "publish" || $post_status == "future") {
	               return;
	            }
            }

            include( OASISWF_PATH . "includes/pages/subpages/submit-workflow.php" );
            $this->enqueue_and_localize_submit_workflow_script();
         } else {
            if ( current_user_can( 'ow_sign_off_step' ) && is_numeric( $chkResult ) &&
                    is_admin() && preg_match_all( '/page=oasiswf(.*)|post-new\.(.*)|post\.(.*)/', $_SERVER['REQUEST_URI'], $matches ) ) {
               include( OASISWF_PATH . "includes/pages/subpages/submit-step.php" );
               $this->enqueue_and_localize_submit_step_script();

               $inbox_service->enqueue_and_localize_script();
            }
         }

         $role = OW_Utility::instance()->get_current_user_role();
         $post_type = get_post_type();
         $post_status = "";

         // do not hide the ootb publish section for skip_workflow_roles option, but hide it if the post is in the workflow
         $show_workflow_for_post_types = get_option( 'oasiswf_show_wfsettings_on_post_types' );
         $row = null;
         if ( isset( $_GET['post'] ) && sanitize_text_field( $_GET["post"] ) && isset( $_GET['action'] ) && sanitize_text_field( $_GET["action"] ) == "edit" ) {
            $post_id = intval( sanitize_text_field( $_GET["post"] ) );
            $post_status = get_post_status( $post_id );
            $row = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM " . OW_Utility::instance()->get_action_history_table_name() . " WHERE post_id = %d AND action_status = 'assignment'", $post_id ) );
         }

         $hide_ootb_publish_section = array(
             "skip_workflow_roles" => true,
             "show_workflow_for_post_types" => true
         );

         if ( current_user_can( 'ow_skip_workflow' ) ) { // do not hide the ootb publish section for skip_workflow_roles option
            $hide_ootb_publish_section["skip_workflow_roles"] = false;
         } else {
            $hide_ootb_publish_section["skip_workflow_roles"] = true;
         }

         if ( is_array( $show_workflow_for_post_types ) && in_array( $post_type, $show_workflow_for_post_types ) ) { // do not show ootb publish section for oasiswf_show_wfsettings_on_post_types
            $hide_ootb_publish_section['show_workflow_for_post_types'] = true;
         } else {
            $hide_ootb_publish_section['show_workflow_for_post_types'] = false;
         }

         // we are dealing with unpublished post
         if ( $hide_ootb_publish_section["skip_workflow_roles"] == 1 &&
                 $hide_ootb_publish_section['show_workflow_for_post_types'] == 1 ) {
            $this->ootb_publish_section_hide();
         }


         // if the item is in the workflow, hide the OOTB publish section
         if ( $row ) {
            $this->ootb_publish_section_hide();
         }

         // Add nonce to the post page
         echo "<input type='hidden' name='owf_claim_process_ajax_nonce' id='owf_claim_process_ajax_nonce' value='" . wp_create_nonce( 'owf_claim_process_ajax_nonce' ) . "'/>";

         wp_nonce_field( 'owf_exit_post_from_workflow_ajax_nonce', 'owf_exit_post_from_workflow' );

         //--------generate abort workflow link---------

         if ( current_user_can( 'ow_abort_workflow' ) ) {
            if ( $row ) {
               echo "<script type='text/javascript'>var exit_wfid = $row->ID ;</script>";
               $this->enqueue_and_localize_abort_script();
            }
         }
      }
   }

   /*
    * When a post is trashed, delete all the action history related to the post
    *
    * @since 2.0
    */

   public function when_post_trash_delete( $post_id ) {
      global $wpdb;

      $post_id = intval( $post_id );

      $ow_history_service = new OW_History_Service();
      $histories = $ow_history_service->get_action_history_by_post( $post_id );
      if ( $histories ) {
         foreach ( $histories as $history ) {
            $wpdb->get_results( $wpdb->prepare( "DELETE FROM " . OW_Utility::instance()->get_action_table_name() . " WHERE action_history_id = %d", $history->ID ) );
            $wpdb->get_results( $wpdb->prepare( "DELETE FROM " . OW_Utility::instance()->get_emails_table_name() . " WHERE history_id = %d", $history->ID ) );
         }
         $wpdb->get_results( $wpdb->prepare( "DELETE FROM " . OW_Utility::instance()->get_action_history_table_name() . " WHERE post_id = %d", $post_id ) );
      }

      $original_post = get_post_meta( $post_id, 'oasis_original', true );
      delete_post_meta( $post_id, 'oasis_original' );
      $this->cleanup_after_workflow_complete( $post_id );
   }


   private function get_unsubmitted_posts( $post_status_list, $post_count ) {
      global $wpdb;

      $post_count = intval( $post_count );

      $all_unsubmitted_posts = array();
      $post_type_condition = "";
      $show_workflow_for_post_types = get_option( 'oasiswf_show_wfsettings_on_post_types' );
      if ( is_array( $show_workflow_for_post_types ) ) {
         $show_workflow_for_post_types_list = "'" . implode( "', '", $show_workflow_for_post_types ) . "'";
         $post_type_condition = " posts.post_type in (" . $show_workflow_for_post_types_list . ") AND ";
      }

      $all_unsubmitted_posts = $wpdb->get_results( "SELECT distinct posts.ID, posts.post_title FROM {$wpdb->prefix}posts posts
		WHERE posts.post_status in (" . $post_status_list . ")
	   	AND " .
              $post_type_condition .
              "(NOT EXISTS (SELECT * from {$wpdb->prefix}postmeta postmeta1 WHERE postmeta1.meta_key = 'oasis_is_in_workflow' and posts.ID = postmeta1.post_id) OR
		   	EXISTS (SELECT * from {$wpdb->prefix}postmeta postmeta2 WHERE postmeta2.meta_key = 'oasis_is_in_workflow' AND postmeta2.meta_value = '0' and posts.ID = postmeta2.post_id))
		   	order by post_modified_gmt
		   	limit 0, " . $post_count );

      return $all_unsubmitted_posts;
   }

   public function owf_submit_to_workflow_hook_test( $post_id, $workflow_history_id ) {
      OW_Utility::instance()->logger( "this is testing the submit to workflow hook" );
      OW_Utility::instance()->logger( "PostID:" . $post_id );
      OW_Utility::instance()->logger( "Workflow History ID:" . $workflow_history_id );
   }

   public function owf_step_sign_off_hook_test( $post_id, $workflow_history_id ) {
      OW_Utility::instance()->logger( "this is testing the step sign off workflow hook" );
      OW_Utility::instance()->logger( "PostID:" . $post_id );
      OW_Utility::instance()->logger( "Workflow History ID:" . $workflow_history_id );
   }

   public function owf_workflow_complete_hook_test( $post_id, $workflowId ) {
      OW_Utility::instance()->logger( "this is testing the workflow completion hook" );
      OW_Utility::instance()->logger( "PostID:" . $post_id );
      OW_Utility::instance()->logger( "Workflow ID:" . $workflowId );
   }

   /*
    * localize submit workflow scripts
    */

   public function enqueue_and_localize_submit_workflow_script() {
      wp_nonce_field( 'owf_workflow_abort_nonce', 'owf_workflow_abort_nonce' );

		// enqueue js file if advanced custom fields plugin active
      $this->enqueue_acf_validator_script();
      wp_enqueue_script( 'owf_submit_workflow', OASISWF_URL . 'js/pages/subpages/submit-workflow.js', array( 'jquery' ), OASISWF_VERSION, true );

      $workflow_terminology_options = get_option( 'oasiswf_custom_workflow_terminology' );
      $submit_to_workflow_label = !empty( $workflow_terminology_options['submitToWorkflowText'] ) ? $workflow_terminology_options['submitToWorkflowText'] : __( 'Submit to Workflow', 'oasisworkflow' );
      wp_localize_script( 'owf_submit_workflow', 'owf_submit_workflow_vars', array(
          'submitToWorkflowButton' => $submit_to_workflow_label,
          'allStepsNotDefined' => __( 'All steps are not defined.\n Please check the workflow.', 'oasisworkflow' ),
          'clickHereText' => __( 'click here', 'oasisworkflow' ),
          'notValidWorkflow' => __( 'The selected workflow is not valid.\n Please check this workflow.', 'oasisworkflow' ),
          'noUsersDefined' => __( 'No users found for the given role.', 'oasisworkflow' ),
          'multipleUsers' => __( 'You can select multiple users only for review step. Selected step is', 'oasisworkflow' ),
          'step' => __( 'step.', 'oasisworkflow' ),
          'selectWorkflow' => __( 'Please select a workflow.', 'oasisworkflow' ),
          'selectStep' => __( 'Please select a step.', 'oasisworkflow' ),
          'stepNotDefined' => __( 'This step is not defined.', 'oasisworkflow' ),
          'dueDateRequired' => __( 'Please enter a due date.', 'oasisworkflow' ),
          'noAssignedActors' => __( 'No assigned actor(s).', 'oasisworkflow' ),
          'drdb' => get_option( 'oasiswf_reminder_days' ),
          'drda' => get_option( 'oasiswf_reminder_days_after' ),
          'allowedPostTypes' => json_encode( get_option( 'oasiswf_show_wfsettings_on_post_types' ) ),
          'dateFormat' => OW_Utility::instance()->owf_date_format_to_jquery_ui_format( get_option( 'date_format' ) ),
          'editDateFormat' => OW_Utility::instance()->owf_date_format_to_jquery_ui_format( OASISWF_EDIT_DATE_FORMAT ),
          'defaultDueDays' => get_option( 'oasiswf_default_due_days' ),
      	 'absoluteURL' => get_admin_url()
      ) );
      //}
   }

   /*
    * localize submit step scripts
    */
   public function enqueue_and_localize_submit_step_script() {
      wp_nonce_field( 'owf_workflow_abort_nonce', 'owf_workflow_abort_nonce' );
      wp_nonce_field( 'owf_check_claim_nonce', 'owf_check_claim_nonce' );

		// enqueue js file if advanced custom fields plugin active
      $this->enqueue_acf_validator_script();

      wp_enqueue_script( 'owf_submit_step', OASISWF_URL . 'js/pages/subpages/submit-step.js', array( 'jquery' ), OASISWF_VERSION, true );

      $workflow_terminology_options = get_option( 'oasiswf_custom_workflow_terminology' );
      $sign_off_label = !empty( $workflow_terminology_options['signOffText'] ) ? $workflow_terminology_options['signOffText'] : __( 'Sign Off', 'oasisworkflow' );
      wp_localize_script( 'owf_submit_step', 'owf_submit_step_vars', array(
          'clickHereText' => __( 'click here', 'oasisworkflow' ),
          'signOffButton' => $sign_off_label,
          'claimButton' => __( 'Claim', 'oasisworkflow' ),
          'inboxButton' => __( 'Go to Workflow Inbox', 'oasisworkflow' ),
          'firstStepMessage' => __( 'This is the first step in the workflow.</br> Do you really want to cancel the post/page from the workflow?', 'oasisworkflow' ),
          'lastStepMessage' => __( 'This is the last step in the workflow. Are you sure to complete the workflow?', 'oasisworkflow' ),
          'noUsersFound' => __( 'No users found for the given role.', 'oasisworkflow' ),
          'decisionSelectMessage' => __( 'Please select an action.', 'oasisworkflow' ),
          'selectStep' => __( 'Please select a step.', 'oasisworkflow' ),
          'dueDateRequired' => __( 'Please enter a due date.', 'oasisworkflow' ),
          'noAssignedActors' => __( 'No assigned actor(s).', 'oasisworkflow' ),
          'multipleUsers' => __( 'You can select multiple users only for review step. Selected step is', 'oasisworkflow' ),
          'step' => __( 'step.', 'oasisworkflow' ),
          'drdb' => get_option( 'oasiswf_reminder_days' ),
          'drda' => get_option( 'oasiswf_reminder_days_after' ),
          'dateFormat' => OW_Utility::instance()->owf_date_format_to_jquery_ui_format( get_option( 'date_format' ) ),
          'editDateFormat' => OW_Utility::instance()->owf_date_format_to_jquery_ui_format( OASISWF_EDIT_DATE_FORMAT ),
          'defaultDueDays' => get_option( 'oasiswf_default_due_days' ),
      	 'absoluteURL' => get_admin_url()
      ) );
   }

   /**
    * include/invoke ACF validation during the workflow submit and sign off process,
    * if ACF plugin is installed and active.
    *
    * @since 2.0
    */
   public function enqueue_acf_validator_script() {
      if ( class_exists( 'acf_pro' ) ) { // applicable to pro version of ACF
         wp_enqueue_script( 'owf_acf_validator',
                 OASISWF_URL . 'js/pages/acf-pro-validator.js',
                 array( 'jquery' ), OASISWF_VERSION, true );
      } else if ( class_exists( 'acf' ) ) { //applicable to free version of ACF
         wp_enqueue_script( 'owf_acf_validator',
                 OASISWF_URL . 'js/pages/acf-validator.js',
                 array( 'jquery' ), OASISWF_VERSION, true );
      }
   }

   /**
    * enqueue and localize the workflow abort script
    *
    * @since 2.0
    */
   public function enqueue_and_localize_abort_script() {
      wp_nonce_field( 'owf_workflow_abort_nonce', 'owf_workflow_abort_nonce' );

      wp_enqueue_script( 'owf-abort-workflow', OASISWF_URL . 'js/pages/subpages/exit.js', '', OASISWF_VERSION, true );

      $workflow_terminology_options = get_option( 'oasiswf_custom_workflow_terminology' );
      $abort_workflow_label = !empty( $workflow_terminology_options['abortWorkflowText'] ) ? $workflow_terminology_options['abortWorkflowText'] : __( 'Abort Workflow', 'oasisworkflow' );

      wp_localize_script( 'owf-abort-workflow', 'owf_abort_workflow_vars', array(
          'clickHereText' => __( 'click here', 'oasisworkflow' ),
          'abortWorkflow' => $abort_workflow_label,
          'abortWorkflowConfirm' => __( 'Are you sure to terminate the workflow?', 'oasisworkflow' ),
      	 'absoluteURL' => get_admin_url()
      ) );
   }

   private function get_original_post( $post_id ) {
      $original_post_id = get_post_meta( $post_id, 'oasis_original', true );
      if ( empty( $original_post_id ) ) {
         return null; // we are probably dealing with an incorrect article
      }

      return $original_post_id;
   }

   private function set_loading_post_status() {
      $status_info = sanitize_text_field( $_POST["hi_process_info"] );
      $post_id = intval( sanitize_text_field( $_POST["post_ID"] ) );
      $temp_array = explode( "@#@", $status_info );
      $ow_history_service = new OW_History_Service();
      $action = $ow_history_service->get_action_history_by_id( $temp_array[0] );
      $this->copy_step_status_to_post( $post_id, $action->step_id, $temp_array[1] );
   }

}

// construct an instance so that the actions get loaded
$ow_process_flow = new OW_Process_Flow();
add_action( 'admin_footer', array( $ow_process_flow, 'step_signoff_popup_setup' ) );
add_filter( 'redirect_post_location', array( $ow_process_flow, 'workflow_submit_action' ), '', 2 );
add_filter( 'get_edit_post_link', array( $ow_process_flow, 'oasis_edit_post_link' ), '', 3 );
?>